@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -87,6 +86,23 @@
     Version string that a plugin must match to fill this dependency.
 */
 
+/*!
+    \variable ExtensionSystem::PluginDependency::type
+    Defines whether the dependency is required or optional.
+    \sa ExtensionSystem::PluginDependency::Type
+*/
+
+/*!
+    \enum ExtensionSystem::PluginDependency::Type
+    Whether the dependency is required or optional.
+    \value Required
+           Dependency needs to be there.
+    \value Optional
+           Dependency is not necessarily needed. You need to make sure that
+           the plugin is able to load without this dependency installed, so
+           for example you may not link to the dependency's library.
+*/
+
 /*!
     \class ExtensionSystem::PluginSpec
     \brief Contains the information of the plugins xml description file and
@@ -127,16 +143,26 @@
     \value Deleted
             The plugin instance has been deleted.
 */
+
 using namespace ExtensionSystem;
 using namespace ExtensionSystem::Internal;
 
 /*!
-    \fn bool PluginDependency::operator==(const PluginDependency &other)
+    \fn uint qHash(const ExtensionSystem::PluginDependency &value)
     \internal
 */
-bool PluginDependency::operator==(const PluginDependency &other)
+uint ExtensionSystem::qHash(const ExtensionSystem::PluginDependency &value)
 {
-    return name == other.name && version == other.version;
+    return qHash(value.name);
+}
+
+/*!
+    \fn bool PluginDependency::operator==(const PluginDependency &other) const
+    \internal
+*/
+bool PluginDependency::operator==(const PluginDependency &other) const
+{
+    return name == other.name && version == other.version && type == other.type;
 }
 
 /*!
@@ -395,22 +421,11 @@ IPlugin *PluginSpec::plugin() const
 
     \sa PluginSpec::dependencies()
 */
-QList<PluginSpec *> PluginSpec::dependencySpecs() const
+QHash<PluginDependency, PluginSpec *> PluginSpec::dependencySpecs() const
 {
     return d->dependencySpecs;
 }
 
-/*!
-    \fn QList<PluginSpec *> PluginSpec::providesForSpecs() const
-    Returns the list of plugins that depend on this one.
-
-    \sa PluginSpec::dependencySpecs()
-*/
-QList<PluginSpec *> PluginSpec::providesForSpecs() const
-{
-    return d->providesSpecs;
-}
-
 //==========PluginSpecPrivate==================
 
 namespace {
@@ -429,6 +444,9 @@ namespace {
     const char * const DEPENDENCY = "dependency";
     const char * const DEPENDENCY_NAME = "name";
     const char * const DEPENDENCY_VERSION = "version";
+    const char * const DEPENDENCY_TYPE = "type";
+    const char * const DEPENDENCY_TYPE_SOFT = "optional";
+    const char * const DEPENDENCY_TYPE_HARD = "required";
     const char * const ARGUMENTLIST = "argumentList";
     const char * const ARGUMENT = "argument";
     const char * const ARGUMENT_NAME = "name";
@@ -471,10 +489,9 @@ bool PluginSpecPrivate::read(const QString &fileName)
     errorString = "";
     dependencies.clear();
     QFile file(fileName);
-    if (!file.exists())
-        return reportError(tr("File does not exist: %1").arg(file.fileName()));
     if (!file.open(QIODevice::ReadOnly))
-        return reportError(tr("Could not open file for read: %1").arg(file.fileName()));
+        return reportError(tr("Cannot open file %1 for reading: %2")
+                           .arg(QDir::toNativeSeparators(file.fileName()), file.errorString()));
     QFileInfo fileInfo(file);
     location = fileInfo.absolutePath();
     filePath = fileInfo.absoluteFilePath();
@@ -726,6 +743,18 @@ void PluginSpecPrivate::readDependencyEntry(QXmlStreamReader &reader)
         reader.raiseError(msgInvalidFormat(DEPENDENCY_VERSION));
         return;
     }
+    dep.type = PluginDependency::Required;
+    if (reader.attributes().hasAttribute(DEPENDENCY_TYPE)) {
+        QString typeValue = reader.attributes().value(DEPENDENCY_TYPE).toString();
+        if (typeValue == QLatin1String(DEPENDENCY_TYPE_HARD)) {
+            dep.type = PluginDependency::Required;
+        } else if (typeValue == QLatin1String(DEPENDENCY_TYPE_SOFT)) {
+            dep.type = PluginDependency::Optional;
+        } else {
+            reader.raiseError(msgInvalidFormat(DEPENDENCY_TYPE));
+            return;
+        }
+    }
     dependencies.append(dep);
     reader.readNext();
     if (reader.tokenType() != QXmlStreamReader::EndElement)
@@ -801,26 +830,27 @@ bool PluginSpecPrivate::resolveDependencies(const QList<PluginSpec *> &specs)
         hasError = true;
         return false;
     }
-    QList<PluginSpec *> resolvedDependencies;
+    QHash<PluginDependency, PluginSpec *> resolvedDependencies;
     foreach (const PluginDependency &dependency, dependencies) {
         PluginSpec *found = 0;
 
         foreach (PluginSpec *spec, specs) {
             if (spec->provides(dependency.name, dependency.version)) {
                 found = spec;
-                spec->d->addProvidesForPlugin(q);
                 break;
             }
         }
         if (!found) {
-            hasError = true;
-            if (!errorString.isEmpty())
-                errorString.append(QLatin1Char('\n'));
-            errorString.append(QCoreApplication::translate("PluginSpec", "Could not resolve dependency '%1(%2)'")
-                .arg(dependency.name).arg(dependency.version));
+            if (dependency.type == PluginDependency::Required) {
+                hasError = true;
+                if (!errorString.isEmpty())
+                    errorString.append(QLatin1Char('\n'));
+                errorString.append(QCoreApplication::translate("PluginSpec", "Could not resolve dependency '%1(%2)'")
+                    .arg(dependency.name).arg(dependency.version));
+            }
             continue;
         }
-        resolvedDependencies.append(found);
+        resolvedDependencies.insert(dependency, found);
     }
     if (hasError)
         return false;
@@ -840,7 +870,12 @@ void PluginSpecPrivate::disableIndirectlyIfDependencyDisabled()
     if (disabledIndirectly)
         return;
 
-    foreach (PluginSpec *dependencySpec, dependencySpecs) {
+    QHashIterator<PluginDependency, PluginSpec *> it(dependencySpecs);
+    while (it.hasNext()) {
+        it.next();
+        if (it.key().type == PluginDependency::Optional)
+            continue;
+        PluginSpec *dependencySpec = it.value();
         if (dependencySpec->isDisabledIndirectly() || !dependencySpec->isEnabled()) {
             disabledIndirectly = true;
             break;
@@ -984,17 +1019,3 @@ void PluginSpecPrivate::kill()
     plugin = 0;
     state = PluginSpec::Deleted;
 }
-
-/*!
-    \fn void PluginSpecPrivate::addProvidesForPlugin(PluginSpec *dependent)
-    \internal
-*/
-void PluginSpecPrivate::addProvidesForPlugin(PluginSpec *dependent)
-{
-    providesSpecs.append(dependent);
-}
-
-void PluginSpecPrivate::removeProvidesForPlugin(PluginSpec *dependent)
-{
-    providesSpecs.removeOne(dependent);
-}
