@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -44,7 +43,10 @@
 #include "cmakeprojectmanager.h"
 
 #include <utils/pathchooser.h>
+#include <projectexplorer/toolchainmanager.h>
 #include <projectexplorer/toolchain.h>
+#include <projectexplorer/abi.h>
+#include <texteditor/fontsettings.h>
 
 #include <QtGui/QVBoxLayout>
 #include <QtGui/QFormLayout>
@@ -72,7 +74,8 @@ CMakeOpenProjectWizard::CMakeOpenProjectWizard(CMakeManager *cmakeManager, const
     : m_cmakeManager(cmakeManager),
       m_sourceDirectory(sourceDirectory),
       m_creatingCbpFiles(false),
-      m_environment(env)
+      m_environment(env),
+      m_toolChain(0)
 {
     int startid;
     if (hasInSourceBuild()) {
@@ -106,7 +109,8 @@ CMakeOpenProjectWizard::CMakeOpenProjectWizard(CMakeManager *cmakeManager, const
     : m_cmakeManager(cmakeManager),
       m_sourceDirectory(sourceDirectory),
       m_creatingCbpFiles(true),
-      m_environment(env)
+      m_environment(env),
+      m_toolChain(0)
 {
 
     CMakeRunPage::Mode rmode;
@@ -126,7 +130,8 @@ CMakeOpenProjectWizard::CMakeOpenProjectWizard(CMakeManager *cmakeManager, const
     : m_cmakeManager(cmakeManager),
       m_sourceDirectory(sourceDirectory),
       m_creatingCbpFiles(true),
-      m_environment(env)
+      m_environment(env),
+      m_toolChain(0)
 {
     m_buildDirectory = oldBuildDirectory;
     addPage(new ShadowBuildPage(this, true));
@@ -196,26 +201,27 @@ void CMakeOpenProjectWizard::setBuildDirectory(const QString &directory)
     m_buildDirectory = directory;
 }
 
-QString CMakeOpenProjectWizard::msvcVersion() const
-{
-    return m_msvcVersion;
-}
-
-void CMakeOpenProjectWizard::setMsvcVersion(const QString &version)
-{
-    m_msvcVersion = version;
-}
-
-QStringList CMakeOpenProjectWizard::arguments() const
+QString CMakeOpenProjectWizard::arguments() const
 {
     return m_arguments;
 }
 
-void CMakeOpenProjectWizard::setArguments(const QStringList &args)
+void CMakeOpenProjectWizard::setArguments(const QString &args)
 {
     m_arguments = args;
 }
 
+ProjectExplorer::ToolChain *CMakeOpenProjectWizard::toolChain() const
+{
+    return m_toolChain;
+}
+
+void CMakeOpenProjectWizard::setToolChain(ProjectExplorer::ToolChain *tc)
+{
+    m_toolChain = tc;
+}
+
+
 Utils::Environment CMakeOpenProjectWizard::environment() const
 {
     return m_environment;
@@ -236,7 +242,6 @@ InSourceBuildPage::InSourceBuildPage(CMakeOpenProjectWizard *cmakeWizard)
     setTitle(tr("Build Location"));
 }
 
-
 ShadowBuildPage::ShadowBuildPage(CMakeOpenProjectWizard *cmakeWizard, bool change)
     : QWizardPage(cmakeWizard), m_cmakeWizard(cmakeWizard)
 {
@@ -252,7 +257,7 @@ ShadowBuildPage::ShadowBuildPage(CMakeOpenProjectWizard *cmakeWizard, bool chang
                           "Qt Creator recommends to not use the source directory for building. "
                           "This ensures that the source directory remains clean and enables multiple builds "
                           "with different settings."));
-    fl->addWidget(label);
+    fl->addRow(label);
     m_pc = new Utils::PathChooser(this);
     m_pc->setBaseDirectory(m_cmakeWizard->sourceDirectory());
     m_pc->setPath(m_cmakeWizard->buildDirectory());
@@ -279,6 +284,7 @@ CMakeRunPage::CMakeRunPage(CMakeOpenProjectWizard *cmakeWizard, Mode mode, const
 void CMakeRunPage::initWidgets()
 {
     QFormLayout *fl = new QFormLayout;
+    fl->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
     setLayout(fl);
     // Description Label
     m_descriptionLabel = new QLabel(this);
@@ -289,50 +295,64 @@ void CMakeRunPage::initWidgets()
     if (m_cmakeWizard->cmakeManager()->isCMakeExecutableValid()) {
         m_cmakeExecutable = 0;
     } else {
-        QString text = tr("Please specify the path to the CMake executable. No CMake executable was found in the path.");
+        QString text = tr("Please specify the path to the cmake executable. No cmake executable was found in the path.");
         QString cmakeExecutable = m_cmakeWizard->cmakeManager()->cmakeExecutable();
         if (!cmakeExecutable.isEmpty()) {
             QFileInfo fi(cmakeExecutable);
             if (!fi.exists())
-                text += tr(" The CMake executable (%1) does not exist.").arg(cmakeExecutable);
+                text += tr(" The cmake executable (%1) does not exist.").arg(cmakeExecutable);
             else if (!fi.isExecutable())
                 text += tr(" The path %1 is not a executable.").arg(cmakeExecutable);
             else
-                text += tr(" The path %1 is not a valid CMake.").arg(cmakeExecutable);
+                text += tr(" The path %1 is not a valid cmake.").arg(cmakeExecutable);
         }
 
-        fl->addRow(new QLabel(text, this));
+        QLabel *cmakeLabel = new QLabel(text);
+        cmakeLabel->setWordWrap(true);
+        fl->addRow(cmakeLabel);
         // Show a field for the user to enter
         m_cmakeExecutable = new Utils::PathChooser(this);
         m_cmakeExecutable->setExpectedKind(Utils::PathChooser::ExistingCommand);
-        fl->addRow("CMake Executable", m_cmakeExecutable);
+        fl->addRow("cmake Executable:", m_cmakeExecutable);
     }
 
     // Run CMake Line (with arguments)
     m_argumentsLineEdit = new QLineEdit(this);
     connect(m_argumentsLineEdit,SIGNAL(returnPressed()), this, SLOT(runCMake()));
+    fl->addRow(tr("Arguments:"), m_argumentsLineEdit);
 
     m_generatorComboBox = new QComboBox(this);
+    fl->addRow(tr("Generator:"), m_generatorComboBox);
 
     m_runCMake = new QPushButton(this);
     m_runCMake->setText(tr("Run CMake"));
     connect(m_runCMake, SIGNAL(clicked()), this, SLOT(runCMake()));
 
-    QHBoxLayout *hbox = new QHBoxLayout;
-    hbox->addWidget(m_argumentsLineEdit);
-    hbox->addWidget(m_generatorComboBox);
-    hbox->addWidget(m_runCMake);
-
-    fl->addRow(tr("Arguments"), hbox);
+    QHBoxLayout *hbox2 = new QHBoxLayout;
+    hbox2->addStretch(10);
+    hbox2->addWidget(m_runCMake);
+    fl->addRow(hbox2);
 
     // Bottom output window
     m_output = new QPlainTextEdit(this);
     m_output->setReadOnly(true);
+    // set smaller minimum size to avoid vanishing descriptions if all of the
+    // above is shown and the dialog not vertically resizing to fit stuff in (Mac)
+    m_output->setMinimumHeight(15);
+    QFont f(TextEditor::FontSettings::defaultFixedFontFamily());
+    f.setStyleHint(QFont::TypeWriter);
+    m_output->setFont(f);
     QSizePolicy pl = m_output->sizePolicy();
     pl.setVerticalStretch(1);
     m_output->setSizePolicy(pl);
     fl->addRow(m_output);
+
+    m_exitCodeLabel = new QLabel(this);
+    m_exitCodeLabel->setVisible(false);
+    fl->addRow(m_exitCodeLabel);
+
     setTitle(tr("Run CMake"));
+    setMinimumSize(600, 400);
 }
 
 void CMakeRunPage::initializePage()
@@ -344,7 +364,7 @@ void CMakeRunPage::initializePage()
         if (m_cmakeWizard->existsUpToDateXmlFile()) {
             m_descriptionLabel->setText(
                     tr("The directory %1 already contains a cbp file, which is recent enough. "
-                       "You can pass special arguments or change the used toolchain here and rerun CMake. "
+                       "You can pass special arguments or change the used tool chain here and rerun CMake. "
                        "Or simply finish the wizard directly.").arg(m_buildDirectory));
         } else {
             m_descriptionLabel->setText(
@@ -372,122 +392,170 @@ void CMakeRunPage::initializePage()
     } else if (m_mode == CMakeRunPage::WantToUpdate) {
         m_descriptionLabel->setText(tr("Refreshing cbp file in %1.").arg(m_buildDirectory));
     }
-    if (m_cmakeWizard->cmakeManager()->hasCodeBlocksMsvcGenerator()) {
-        m_generatorComboBox->setVisible(true);
-        QString cachedGenerator;
-        // Try to find out generator from CMakeCachhe file, if it exists
 
-        QFile fi(m_buildDirectory + "/CMakeCache.txt");
-        if (fi.exists()) {
-            // Cache exists, then read it...
-            if (fi.open(QIODevice::ReadOnly | QIODevice::Text)) {
-                while (!fi.atEnd()) {
-                    QString line = fi.readLine();
-                    if (line.startsWith("CMAKE_GENERATOR:INTERNAL=")) {
-                        int splitpos = line.indexOf('=');
-                        if (splitpos != -1) {
-                            cachedGenerator = line.mid(splitpos + 1).trimmed();
-                        }
-                        break;
-                    }
+
+    // Try figuring out generator and toolchain from CMakeCache.txt
+    QString cachedGenerator;
+    QString cmakeCxxCompiler;
+    QFile fi(m_buildDirectory + "/CMakeCache.txt");
+    if (fi.exists()) {
+        // Cache exists, then read it...
+        if (fi.open(QIODevice::ReadOnly | QIODevice::Text)) {
+            while (!fi.atEnd()) {
+                QString line = fi.readLine();
+                if (line.startsWith("CMAKE_GENERATOR:INTERNAL=")) {
+                    int splitpos = line.indexOf('=');
+                    if (splitpos != -1)
+                        cachedGenerator = line.mid(splitpos + 1).trimmed();
                 }
+                if (line.startsWith("CMAKE_CXX_COMPILER:FILEPATH=")) {
+                    int splitpos = line.indexOf("=");
+                    if (splitpos != -1)
+                        cmakeCxxCompiler = line.mid(splitpos +1).trimmed();
+                }
+                if (!cachedGenerator.isEmpty() && !cmakeCxxCompiler.isEmpty())
+                    break;
             }
         }
-        m_generatorComboBox->clear();
-        // Find out whether we have multiple msvc versions
-        QStringList msvcVersions = ProjectExplorer::ToolChain::availableMSVCVersions();
-        if (msvcVersions.isEmpty()) {
+    }
 
-        } else if (msvcVersions.count() == 1) {
-            m_generatorComboBox->addItem(tr("NMake Generator"), msvcVersions.first());
+    // Build the list of generators/toolchains we want to offer
+    // todo restrict toolchains based on CMAKE_CXX_COMPILER ?
+    Q_UNUSED(cmakeCxxCompiler);
+    m_generatorComboBox->clear();
+    bool hasCodeBlocksGenerator = m_cmakeWizard->cmakeManager()->hasCodeBlocksMsvcGenerator();
+    ProjectExplorer::Abi abi = ProjectExplorer::Abi::hostAbi();
+    abi = ProjectExplorer::Abi(abi.architecture(), abi.os(), ProjectExplorer::Abi::UnknownFlavor,
+                               abi.binaryFormat(), 0);
+    QList<ProjectExplorer::ToolChain *> tcs =
+            ProjectExplorer::ToolChainManager::instance()->findToolChains(abi);
+
+    foreach (ProjectExplorer::ToolChain *tc, tcs) {
+        ProjectExplorer::Abi targetAbi = tc->targetAbi();
+        QVariant tcVariant = qVariantFromValue(static_cast<void *>(tc));
+        if (targetAbi.os() == ProjectExplorer::Abi::WindowsOS) {
+            if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2005Flavor
+                    || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2008Flavor
+                    || targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMsvc2010Flavor) {
+                if (hasCodeBlocksGenerator && (cachedGenerator.isEmpty() || cachedGenerator == "NMake Makefiles"))
+                    m_generatorComboBox->addItem(tr("NMake Generator (%1)").arg(tc->displayName()), tcVariant);
+             } else if (targetAbi.osFlavor() == ProjectExplorer::Abi::WindowsMSysFlavor) {
+                if (cachedGenerator.isEmpty() || cachedGenerator == "MinGW Makefiles")
+                    m_generatorComboBox->addItem(tr("MinGW Generator (%1)").arg(tc->displayName()), tcVariant);
+            }
         } else {
-            foreach (const QString &msvcVersion, msvcVersions)
-                m_generatorComboBox->addItem(tr("NMake Generator (%1)").arg(msvcVersion), msvcVersion);
+            // Non windows
+            if (cachedGenerator.isEmpty() || cachedGenerator == "Unix Makefiles")
+                m_generatorComboBox->addItem(tr("Unix Generator (%1)").arg(tc->displayName()), tcVariant);
         }
-
-        if (cachedGenerator == "NMake Makefiles" && !msvcVersions.isEmpty()) {
-            m_generatorComboBox->setCurrentIndex(0);
-            m_cmakeWizard->setMsvcVersion(msvcVersions.first());
-        }
-
-        m_generatorComboBox->addItem(tr("MinGW Generator"), "mingw");
-        if (cachedGenerator == "MinGW Makefiles") {
-            m_generatorComboBox->setCurrentIndex(m_generatorComboBox->count() - 1);
-            m_cmakeWizard->setMsvcVersion("");
-        }
-    } else {
-        // No new enough cmake, simply hide the combo box
-        m_generatorComboBox->setVisible(false);
     }
 }
 
 void CMakeRunPage::runCMake()
 {
-    m_runCMake->setEnabled(false);
-    m_argumentsLineEdit->setEnabled(false);
-    QStringList arguments = Utils::Environment::parseCombinedArgString(m_argumentsLineEdit->text());
-    CMakeManager *cmakeManager = m_cmakeWizard->cmakeManager();
-
-#ifdef Q_OS_WIN
-    m_cmakeWizard->setMsvcVersion(QString());
-    QString generator = QLatin1String("-GCodeBlocks - MinGW Makefiles");
-    if (m_generatorComboBox->isVisible()) {
-         // the combobox is shown, check which generator is selected
-        int index = m_generatorComboBox->currentIndex();
-        if (index != -1) {
-            QString version = m_generatorComboBox->itemData(index).toString();
-            if (version != "mingw") {
-                generator = "-GCodeBlocks - NMake Makefiles";
-                m_cmakeWizard->setMsvcVersion(version);
-            } else {
-                m_cmakeWizard->setMsvcVersion("");
-            }
-        }
-    }
-#else // Q_OS_WIN
-    QString generator = QLatin1String("-GCodeBlocks - Unix Makefiles");
-#endif
-    Utils::Environment env = m_cmakeWizard->environment();
-    if (!m_cmakeWizard->msvcVersion().isEmpty()) {
-        // Add the environment of that msvc version to environment
-        ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChain::createMSVCToolChain(m_cmakeWizard->msvcVersion(), false);
-        tc->addToEnvironment(env);
-        delete tc;
-    }
-
     if (m_cmakeExecutable) {
         // We asked the user for the cmake executable
         m_cmakeWizard->cmakeManager()->setCMakeExecutable(m_cmakeExecutable->path());
     }
 
+    int index = m_generatorComboBox->currentIndex();
+
+    ProjectExplorer::ToolChain *tc = 0;
+    if (index >= 0)
+        tc = static_cast<ProjectExplorer::ToolChain *>(m_generatorComboBox->itemData(index).value<void *>());
+    if (!tc) {
+        m_output->appendPlainText(tr("No generator selected."));
+        return;
+    }
+
+    m_cmakeWizard->setToolChain(tc);
+
+    m_runCMake->setEnabled(false);
+    m_argumentsLineEdit->setEnabled(false);
+    m_generatorComboBox->setEnabled(false);
+    CMakeManager *cmakeManager = m_cmakeWizard->cmakeManager();
+
+    QString generator = QLatin1String("-GCodeBlocks - Unix Makefiles");
+    if (tc->targetAbi().os() == ProjectExplorer::Abi::WindowsOS) {
+        if (tc->targetAbi().osFlavor() == ProjectExplorer::Abi::WindowsMSysFlavor)
+            generator = QLatin1String("-GCodeBlocks - MinGW Makefiles");
+        else
+            generator = QLatin1String("-GCodeBlocks - NMake Makefiles");
+    }
+
+
+    Utils::Environment env = m_cmakeWizard->environment();
+    tc->addToEnvironment(env);
+
+
+
     m_output->clear();
 
     if (m_cmakeWizard->cmakeManager()->isCMakeExecutableValid()) {
-        m_cmakeProcess = new QProcess();
-        connect(m_cmakeProcess, SIGNAL(readyRead()), this, SLOT(cmakeReadyRead()));
+        m_cmakeProcess = new Utils::QtcProcess();
+        connect(m_cmakeProcess, SIGNAL(readyReadStandardOutput()), this, SLOT(cmakeReadyReadStandardOutput()));
+        connect(m_cmakeProcess, SIGNAL(readyReadStandardError()), this, SLOT(cmakeReadyReadStandardError()));
         connect(m_cmakeProcess, SIGNAL(finished(int)), this, SLOT(cmakeFinished()));
-        cmakeManager->createXmlFile(m_cmakeProcess, arguments, m_cmakeWizard->sourceDirectory(), m_buildDirectory, env, generator);
+        cmakeManager->createXmlFile(m_cmakeProcess, m_argumentsLineEdit->text(), m_cmakeWizard->sourceDirectory(), m_buildDirectory, env, generator);
     } else {
         m_runCMake->setEnabled(true);
         m_argumentsLineEdit->setEnabled(true);
-        m_output->appendPlainText(tr("No valid CMake executable specified."));
+        m_generatorComboBox->setEnabled(true);
+        m_output->appendPlainText(tr("No valid cmake executable specified."));
     }
 }
 
-void CMakeRunPage::cmakeReadyRead()
+static QColor mix_colors(QColor a, QColor b)
 {
-    m_output->appendPlainText(m_cmakeProcess->readAll());
+    return QColor((a.red() + 2 * b.red()) / 3, (a.green() + 2 * b.green()) / 3,
+                  (a.blue() + 2* b.blue()) / 3, (a.alpha() + 2 * b.alpha()) / 3);
+}
+
+void CMakeRunPage::cmakeReadyReadStandardOutput()
+{
+    QTextCursor cursor(m_output->document());
+    cursor.movePosition(QTextCursor::End);
+    QTextCharFormat tf;
+
+    QFont font = m_output->font();
+    tf.setFont(font);
+    tf.setForeground(m_output->palette().color(QPalette::Text));
+
+    cursor.insertText(m_cmakeProcess->readAllStandardOutput(), tf);
+}
+
+void CMakeRunPage::cmakeReadyReadStandardError()
+{
+    QTextCursor cursor(m_output->document());
+    QTextCharFormat tf;
+
+    QFont font = m_output->font();
+    QFont boldFont = font;
+    boldFont.setBold(true);
+    tf.setFont(boldFont);
+    tf.setForeground(mix_colors(m_output->palette().color(QPalette::Text), QColor(Qt::red)));
+
+    cursor.insertText(m_cmakeProcess->readAllStandardError(), tf);
 }
 
 void CMakeRunPage::cmakeFinished()
 {
     m_runCMake->setEnabled(true);
     m_argumentsLineEdit->setEnabled(true);
+    m_generatorComboBox->setEnabled(true);
+
+    if (m_cmakeProcess->exitCode() != 0) {
+        m_exitCodeLabel->setVisible(true);
+        m_exitCodeLabel->setText(tr("CMake exited with errors. Please check CMake output."));
+        m_complete = false;
+    } else {
+        m_exitCodeLabel->setVisible(false);
+        m_complete = true;
+    }
     m_cmakeProcess->deleteLater();
     m_cmakeProcess = 0;
-    m_cmakeWizard->setArguments(Utils::Environment::parseCombinedArgString(m_argumentsLineEdit->text()));
+    m_cmakeWizard->setArguments(m_argumentsLineEdit->text());
     //TODO Actually test that running cmake was finished, for setting this bool
-    m_complete = true;
     emit completeChanged();
 }
 
@@ -495,6 +563,7 @@ void CMakeRunPage::cleanupPage()
 {
     m_output->clear();
     m_complete = false;
+    m_exitCodeLabel->setVisible(false);
     emit completeChanged();
 }
 
@@ -502,4 +571,3 @@ bool CMakeRunPage::isComplete() const
 {
     return m_complete;
 }
-
