@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -38,10 +37,14 @@
 #include "coreimpl.h"
 #include "coreconstants.h"
 #include "editormanager.h"
+#include "externaltool.h"
+#include "toolsettings.h"
+#include "mimetypesettings.h"
 #include "fancytabwidget.h"
 #include "filemanager.h"
 #include "generalsettings.h"
 #include "helpmanager.h"
+#include "ieditor.h"
 #include "ifilefactory.h"
 #include "messagemanager.h"
 #include "modemanager.h"
@@ -54,14 +57,14 @@
 #include "progressview.h"
 #include "shortcutsettings.h"
 #include "vcsmanager.h"
-#include "ieditor.h"
+#include "variablechooser.h"
 
 #include "scriptmanager_p.h"
 #include "settingsdialog.h"
 #include "variablemanager.h"
 #include "versiondialog.h"
 #include "statusbarmanager.h"
-#include "uniqueidmanager.h"
+#include "id.h"
 #include "manhattanstyle.h"
 #include "navigationwidget.h"
 #include "rightpane.h"
@@ -69,8 +72,10 @@
 #include "statusbarwidget.h"
 #include "basefilewizard.h"
 #include "ioutputpane.h"
+#include "externaltoolmanager.h"
 #include "editormanager/systemeditor.h"
 
+#include <app/app_version.h>
 #include <coreplugin/findplaceholder.h>
 #include <coreplugin/icorelistener.h>
 #include <coreplugin/inavigationwidgetfactory.h>
@@ -86,6 +91,8 @@
 #include <QtCore/QTimer>
 #include <QtCore/QtPlugin>
 #include <QtCore/QUrl>
+#include <QtCore/QDir>
+#include <QtCore/QFile>
 
 #include <QtGui/QApplication>
 #include <QtGui/QCloseEvent>
@@ -98,6 +105,7 @@
 #include <QtGui/QToolButton>
 #include <QtGui/QMessageBox>
 #include <QtGui/QMenuBar>
+#include <QtGui/QPushButton>
 
 /*
 #ifdef Q_OS_UNIX
@@ -119,11 +127,8 @@ enum { debugMainWindow = 0 };
 MainWindow::MainWindow() :
     EventFilteringMainWindow(),
     m_coreImpl(new CoreImpl(this)),
-    m_uniqueIDManager(new UniqueIDManager()),
     m_additionalContexts(Constants::C_GLOBAL),
-    // keep this in sync with main() in app/main.cpp
-    m_settings(new QSettings(QSettings::IniFormat, QSettings::UserScope,
-                             QLatin1String("Nokia"), QLatin1String("QtCreator"), this)),
+    m_settings(ExtensionSystem::PluginManager::instance()->settings()),
     m_globalSettings(new QSettings(QSettings::IniFormat, QSettings::SystemScope,
                              QLatin1String("Nokia"), QLatin1String("QtCreator"), this)),
     m_settingsDatabase(new SettingsDatabase(QFileInfo(m_settings->fileName()).path(),
@@ -132,11 +137,12 @@ MainWindow::MainWindow() :
     m_printer(0),
     m_actionManager(new ActionManagerPrivate(this)),
     m_editorManager(0),
+    m_externalToolManager(0),
     m_fileManager(new FileManager(this)),
     m_progressManager(new ProgressManagerPrivate()),
     m_scriptManager(new ScriptManagerPrivate(this)),
     m_variableManager(new VariableManager),
-    m_vcsManager(new VCSManager),
+    m_vcsManager(new VcsManager),
     m_statusBarManager(0),
     m_modeManager(0),
     m_mimeDatabase(new MimeDatabase),
@@ -147,6 +153,8 @@ MainWindow::MainWindow() :
     m_activeContext(0),
     m_generalSettings(new GeneralSettings),
     m_shortcutSettings(new ShortcutSettings),
+    m_toolSettings(new ToolSettings),
+    m_mimeTypeSettings(new MimeTypeSettings),
     m_systemEditor(new SystemEditor),
     m_focusToEditor(0),
     m_newAction(0),
@@ -205,6 +213,7 @@ MainWindow::MainWindow() :
     m_messageManager = new MessageManager;
     m_editorManager = new EditorManager(m_coreImpl, this);
     m_editorManager->hide();
+    m_externalToolManager = new ExternalToolManager(m_coreImpl);
     setCentralWidget(m_modeStack);
 
     connect(QApplication::instance(), SIGNAL(focusChanged(QWidget*,QWidget*)),
@@ -249,21 +258,27 @@ MainWindow::~MainWindow()
     ExtensionSystem::PluginManager *pm = ExtensionSystem::PluginManager::instance();
     pm->removeObject(m_shortcutSettings);
     pm->removeObject(m_generalSettings);
+    pm->removeObject(m_toolSettings);
+    pm->removeObject(m_mimeTypeSettings);
     pm->removeObject(m_systemEditor);
+    delete m_externalToolManager;
+    m_externalToolManager = 0;
     delete m_messageManager;
     m_messageManager = 0;
     delete m_shortcutSettings;
     m_shortcutSettings = 0;
     delete m_generalSettings;
     m_generalSettings = 0;
+    delete m_toolSettings;
+    m_toolSettings = 0;
+    delete m_mimeTypeSettings;
+    m_mimeTypeSettings = 0;
     delete m_systemEditor;
     m_systemEditor = 0;
     delete m_settings;
     m_settings = 0;
     delete m_printer;
     m_printer = 0;
-    delete m_uniqueIDManager;
-    m_uniqueIDManager = 0;
     delete m_vcsManager;
     m_vcsManager = 0;
     //we need to delete editormanager and statusbarmanager explicitly before the end of the destructor,
@@ -313,9 +328,10 @@ bool MainWindow::init(QString *errorMessage)
 
     pm->addObject(m_generalSettings);
     pm->addObject(m_shortcutSettings);
+    pm->addObject(m_toolSettings);
+    pm->addObject(m_mimeTypeSettings);
     pm->addObject(m_systemEditor);
 
-
     // Add widget to the bottom, we create the view here instead of inside the
     // OutputPaneManager, since the StatusBarManager needs to be initialized before
     m_outputView = new Core::StatusBarWidget;
@@ -406,7 +422,7 @@ static bool isDesktopFileManagerDrop(const QMimeData *d, QStringList *files = 0)
 
 void MainWindow::dragEnterEvent(QDragEnterEvent *event)
 {
-    if (isDesktopFileManagerDrop(event->mimeData())) {
+    if (isDesktopFileManagerDrop(event->mimeData()) && m_filesToOpenDelayed.isEmpty()) {
         event->accept();
     } else {
         event->ignore();
@@ -418,12 +434,23 @@ void MainWindow::dropEvent(QDropEvent *event)
     QStringList files;
     if (isDesktopFileManagerDrop(event->mimeData(), &files)) {
         event->accept();
-        openFiles(files, ICore::SwitchMode);
+        m_filesToOpenDelayed.append(files);
+        QTimer::singleShot(50, this, SLOT(openDelayedFiles()));
     } else {
         event->ignore();
     }
 }
 
+void MainWindow::openDelayedFiles()
+{
+    if (m_filesToOpenDelayed.isEmpty())
+        return;
+    activateWindow();
+    raise();
+    openFiles(m_filesToOpenDelayed, ICore::SwitchMode);
+    m_filesToOpenDelayed.clear();
+}
+
 IContext *MainWindow::currentContextObject() const
 {
     return m_activeContext;
@@ -547,10 +574,6 @@ void MainWindow::registerDefaultActions()
     cmd = createSeparator(am, this, QLatin1String("QtCreator.Edit.Sep.Advanced"), globalContext);
     medit->addAction(cmd, Constants::G_EDIT_ADVANCED);
 
-    // Tools menu separators
-    cmd = createSeparator(am, this, QLatin1String("QtCreator.Tools.Sep.Options"), globalContext);
-    mtools->addAction(cmd, Constants::G_DEFAULT_THREE);
-
     // Return to editor shortcut: Note this requires Qt to fix up
     // handling of shortcut overrides in menus, item views, combos....
     m_focusToEditor = new QShortcut(this);
@@ -584,10 +607,12 @@ void MainWindow::registerDefaultActions()
     ActionContainer *ac = am->createMenu(Constants::M_FILE_RECENTFILES);
     mfile->addMenu(ac, Constants::G_FILE_OPEN);
     ac->menu()->setTitle(tr("Recent &Files"));
+    ac->setOnAllDisabledBehavior(ActionContainer::Show);
 
     // Save Action
     icon = QIcon::fromTheme(QLatin1String("document-save"), QIcon(Constants::ICON_SAVEFILE));
     QAction *tmpaction = new QAction(icon, tr("&Save"), this);
+    tmpaction->setEnabled(false);
     cmd = am->registerAction(tmpaction, Constants::SAVE, globalContext);
     cmd->setDefaultKeySequence(QKeySequence::Save);
     cmd->setAttribute(Command::CA_UpdateText);
@@ -597,6 +622,7 @@ void MainWindow::registerDefaultActions()
     // Save As Action
     icon = QIcon::fromTheme(QLatin1String("document-save-as"));
     tmpaction = new QAction(icon, tr("Save &As..."), this);
+    tmpaction->setEnabled(false);
     cmd = am->registerAction(tmpaction, Constants::SAVEAS, globalContext);
 #ifdef Q_WS_MAC
     cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+Shift+S")));
@@ -617,6 +643,7 @@ void MainWindow::registerDefaultActions()
     // Print Action
     icon = QIcon::fromTheme(QLatin1String("document-print"));
     tmpaction = new QAction(icon, tr("&Print..."), this);
+    tmpaction->setEnabled(false);
     cmd = am->registerAction(tmpaction, Constants::PRINT, globalContext);
     cmd->setDefaultKeySequence(QKeySequence::Print);
     mfile->addAction(cmd, Constants::G_FILE_PRINT);
@@ -675,7 +702,7 @@ void MainWindow::registerDefaultActions()
 
     // Select All
     icon = QIcon::fromTheme(QLatin1String("edit-select-all"));
-    tmpaction = new QAction(icon, tr("&Select All"), this);
+    tmpaction = new QAction(icon, tr("Select &All"), this);
     cmd = am->registerAction(tmpaction, Constants::SELECTALL, globalContext);
     cmd->setDefaultKeySequence(QKeySequence::SelectAll);
     medit->addAction(cmd, Constants::G_EDIT_SELECTALL);
@@ -683,20 +710,23 @@ void MainWindow::registerDefaultActions()
 
     // Goto Action
     icon = QIcon::fromTheme(QLatin1String("go-jump"));
-    tmpaction = new QAction(icon, tr("&Go To Line..."), this);
+    tmpaction = new QAction(icon, tr("&Go to Line..."), this);
     cmd = am->registerAction(tmpaction, Constants::GOTO, globalContext);
     cmd->setDefaultKeySequence(QKeySequence(tr("Ctrl+L")));
     medit->addAction(cmd, Constants::G_EDIT_OTHER);
     tmpaction->setEnabled(false);
 
     // Options Action
+    mtools->appendGroup(Constants::G_TOOLS_OPTIONS);
+    cmd = createSeparator(am, this, QLatin1String("QtCreator.Tools.Sep.Options"), globalContext);
+    mtools->addAction(cmd, Constants::G_TOOLS_OPTIONS);
     m_optionsAction = new QAction(tr("&Options..."), this);
     cmd = am->registerAction(m_optionsAction, Constants::OPTIONS, globalContext);
 #ifdef Q_WS_MAC
     cmd->setDefaultKeySequence(QKeySequence("Ctrl+,"));
     cmd->action()->setMenuRole(QAction::PreferencesRole);
 #endif
-    mtools->addAction(cmd, Constants::G_DEFAULT_THREE);
+    mtools->addAction(cmd, Constants::G_TOOLS_OPTIONS);
     connect(m_optionsAction, SIGNAL(triggered()), this, SLOT(showOptionsDialog()));
 
 #ifdef Q_WS_MAC
@@ -833,16 +863,20 @@ void MainWindow::openFiles(const QStringList &fileNames, ICore::OpenFilesFlags f
         const QFileInfo fi(fileName);
         const QString absoluteFilePath = fi.absoluteFilePath();
         if (IFileFactory *fileFactory = findFileFactory(nonEditorFileFactories, mimeDatabase(), fi)) {
-            fileFactory->open(absoluteFilePath);
-            if (flags && ICore::SwitchMode)
+            Core::IFile *file = fileFactory->open(absoluteFilePath);
+            if (!file && (flags & ICore::StopOnLoadFail))
+                return;
+            if (file && (flags & ICore::SwitchMode))
                 Core::ModeManager::instance()->activateMode(Core::Constants::MODE_EDIT);
         } else {
             QFlags<EditorManager::OpenEditorFlag> emFlags;
-            if (flags && ICore::SwitchMode)
+            if (flags & ICore::SwitchMode)
                 emFlags = EditorManager::ModeSwitch;
-            if (flags && ICore::CanContainLineNumbers)
+            if (flags & ICore::CanContainLineNumbers)
                 emFlags |=  EditorManager::CanContainLineNumber;
-            editorManager()->openEditor(absoluteFilePath, QString(), emFlags);
+            Core::IEditor *editor = editorManager()->openEditor(absoluteFilePath, QString(), emFlags);
+            if (!editor && (flags & ICore::StopOnLoadFail))
+                return;
         }
     }
 }
@@ -969,8 +1003,8 @@ void MainWindow::openFileWith()
     QStringList fileNames = editorManager()->getOpenFileNames();
     foreach (const QString &fileName, fileNames) {
         bool isExternal;
-        const QString editorId = editorManager()->getOpenWithEditorId(fileName, &isExternal);
-        if (editorId.isEmpty())
+        const Id editorId = editorManager()->getOpenWithEditorId(fileName, &isExternal);
+        if (!editorId.isValid())
             continue;
         if (isExternal) {
             editorManager()->openExternalEditor(fileName, editorId);
@@ -990,17 +1024,12 @@ FileManager *MainWindow::fileManager() const
     return m_fileManager;
 }
 
-UniqueIDManager *MainWindow::uniqueIDManager() const
-{
-    return m_uniqueIDManager;
-}
-
 MessageManager *MainWindow::messageManager() const
 {
     return m_messageManager;
 }
 
-VCSManager *MainWindow::vcsManager() const
+VcsManager *MainWindow::vcsManager() const
 {
     return m_vcsManager;
 }
@@ -1129,15 +1158,12 @@ void MainWindow::updateContextObject(IContext *context)
 {
     if (context == m_activeContext)
         return;
-    IContext *oldContext = m_activeContext;
+    emit m_coreImpl->contextAboutToChange(context);
     m_activeContext = context;
-    if (!context || oldContext != m_activeContext) {
-        emit m_coreImpl->contextAboutToChange(context);
-        updateContext();
-        if (debugMainWindow)
-            qDebug() << "new context object =" << context << (context ? context->widget() : 0)
-                     << (context ? context->widget()->metaObject()->className() : 0);
-    }
+    updateContext();
+    if (debugMainWindow)
+        qDebug() << "new context object =" << context << (context ? context->widget() : 0)
+                 << (context ? context->widget()->metaObject()->className() : 0);
 }
 
 void MainWindow::resetContext()
@@ -1153,11 +1179,15 @@ void MainWindow::aboutToShutdown()
     hide();
 }
 
-static const char *settingsGroup = "MainWindow";
-static const char *geometryKey = "Geometry";
-static const char *colorKey = "Color";
-static const char *maxKey = "Maximized";
-static const char *fullScreenKey = "FullScreen";
+static const char settingsGroup[] = "MainWindow";
+static const char colorKey[] = "Color";
+static const char windowGeometryKey[] = "WindowGeometry";
+static const char windowStateKey[] = "WindowState";
+
+// TODO compat for <= 2.1, remove later
+static const char geometryKey[] = "Geometry";
+static const char maxKey[] = "Maximized";
+static const char fullScreenKey[] = "FullScreen";
 
 void MainWindow::readSettings()
 {
@@ -1173,15 +1203,10 @@ void MainWindow::readSettings()
                                   QColor(Utils::StyleHelper::DEFAULT_BASE_COLOR)).value<QColor>());
     }
 
-    const QVariant geom = m_settings->value(QLatin1String(geometryKey));
-    if (geom.isValid()) {
-        setGeometry(geom.toRect());
-    } else {
-        resize(1024, 700);
+    if (!restoreGeometry(m_settings->value(QLatin1String(windowGeometryKey)).toByteArray())) {
+        resize(1008, 700); // size without window decoration
     }
-    if (m_settings->value(QLatin1String(maxKey), false).toBool())
-        setWindowState(Qt::WindowMaximized);
-    setFullScreen(m_settings->value(QLatin1String(fullScreenKey), false).toBool());
+    restoreState(m_settings->value(QLatin1String(windowStateKey)).toByteArray());
 
     m_settings->endGroup();
 
@@ -1197,18 +1222,12 @@ void MainWindow::writeSettings()
     if (!(m_overrideColor.isValid() && Utils::StyleHelper::baseColor() == m_overrideColor))
         m_settings->setValue(QLatin1String(colorKey), Utils::StyleHelper::requestedBaseColor());
 
-    if (windowState() & (Qt::WindowMaximized | Qt::WindowFullScreen)) {
-        m_settings->setValue(QLatin1String(maxKey), (bool) (windowState() & Qt::WindowMaximized));
-        m_settings->setValue(QLatin1String(fullScreenKey), (bool) (windowState() & Qt::WindowFullScreen));
-    } else {
-        m_settings->setValue(QLatin1String(maxKey), false);
-        m_settings->setValue(QLatin1String(fullScreenKey), false);
-        m_settings->setValue(QLatin1String(geometryKey), geometry());
-    }
+    m_settings->setValue(QLatin1String(windowGeometryKey), saveGeometry());
+    m_settings->setValue(QLatin1String(windowStateKey), saveState());
 
     m_settings->endGroup();
 
-    m_fileManager->saveRecentFiles();
+    m_fileManager->saveSettings();
     m_actionManager->saveSettings(m_settings);
     m_editorManager->saveSettings();
     m_navigationWidget->saveSettings(m_settings);
@@ -1268,22 +1287,29 @@ void MainWindow::aboutToShowRecentFiles()
     aci->menu()->clear();
 
     bool hasRecentFiles = false;
-    foreach (const QString &fileName, m_fileManager->recentFiles()) {
+    foreach (const FileManager::RecentFile &file, m_fileManager->recentFiles()) {
         hasRecentFiles = true;
         QAction *action = aci->menu()->addAction(
-                    Utils::withTildeHomePath(fileName));
-        action->setData(fileName);
+                    QDir::toNativeSeparators(Utils::withTildeHomePath(file.first)));
+        action->setData(qVariantFromValue(file));
         connect(action, SIGNAL(triggered()), this, SLOT(openRecentFile()));
     }
     aci->menu()->setEnabled(hasRecentFiles);
+
+    // add the Clear Menu item
+    if (hasRecentFiles) {
+        aci->menu()->addSeparator();
+        QAction *action = aci->menu()->addAction(QCoreApplication::translate(
+                                                     "Core", Core::Constants::TR_CLEAR_MENU));
+        connect(action, SIGNAL(triggered()), m_fileManager, SLOT(clearRecentFiles()));
+    }
 }
 
 void MainWindow::openRecentFile()
 {
     if (const QAction *action = qobject_cast<const QAction*>(sender())) {
-        const QString fileName = action->data().toString();
-        if (!fileName.isEmpty())
-            editorManager()->openEditor(fileName, QString(), Core::EditorManager::ModeSwitch);
+        const FileManager::RecentFile file = action->data().value<FileManager::RecentFile>();
+        editorManager()->openEditor(file.first, file.second, Core::EditorManager::ModeSwitch);
     }
 }
 
@@ -1348,7 +1374,7 @@ bool MainWindow::showWarningWithOptions(const QString &title,
         parent = this;
     QMessageBox msgBox(QMessageBox::Warning, title, text,
                        QMessageBox::Ok, parent);
-    if (details.isEmpty())
+    if (!details.isEmpty())
         msgBox.setDetailedText(details);
     QAbstractButton *settingsButton = 0;
     if (!settingsId.isEmpty() || !settingsCategory.isEmpty())
