@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -34,12 +33,13 @@
 #include "symbian.h"
 #include "registerhandler.h"
 #include "threadshandler.h"
-#include <trkutils.h>
+#include <codautils.h>
 
 #include <utils/qtcassert.h>
 
 #include <QtCore/QDebug>
 #include <QtCore/QTextStream>
+#include <QtCore/QFileInfo>
 
 namespace Debugger {
 namespace Internal {
@@ -59,7 +59,7 @@ MemoryRange::MemoryRange(uint f, uint t)
 bool MemoryRange::intersects(const MemoryRange &other) const
 {
     Q_UNUSED(other);
-    QTC_ASSERT(false, /**/);
+    QTC_CHECK(false);
     return false; // FIXME
 }
 
@@ -120,7 +120,7 @@ QByteArray dumpRegister(uint n, uint value)
         ba += QByteArray::number(n);
     }
     ba += '=';
-    ba += trk::hexxNumber(value);
+    ba += Coda::hexxNumber(value);
     return ba;
 }
 
@@ -145,8 +145,8 @@ QByteArray Thread::gdbReportRegisters() const
 {
     QByteArray ba;
     for (int i = 0; i < 16; ++i) {
-        const uint reg = trk::swapEndian(registers[i]);
-        ba += trk::hexNumber(reg, 8);
+        const uint reg = Coda::swapEndian(registers[i]);
+        ba += Coda::hexNumber(reg, 8);
     }
     return ba;
 }
@@ -174,18 +174,18 @@ QByteArray Thread::gdbRegisterLogMessage(bool verbose) const
 QByteArray Thread::gdbReportSingleRegister(unsigned i) const
 {
     if (i == RegisterPSGdb)
-        i = RegisterPSTrk;
+        i = RegisterPSCoda;
     if (i >= RegisterCount)
         return QByteArray("0000"); // Unknown
     QByteArray ba;
-    appendInt(&ba, registers[i], trk::LittleEndian);
+    appendInt(&ba, registers[i], Coda::LittleEndian);
     return ba.toHex();
 }
 
 QByteArray Thread::gdbSingleRegisterLogMessage(unsigned i) const
 {
     if (i == RegisterPSGdb)
-        i = RegisterPSTrk;
+        i = RegisterPSCoda;
     if (i >= RegisterCount)
         return QByteArray("Read single unknown register #") + QByteArray::number(i);
     QByteArray logMsg = "Read Register ";
@@ -207,14 +207,12 @@ Snapshot::Snapshot()
 
 void Snapshot::reset()
 {
+    MEMORY_DEBUG("RESET SNAPSHOT MEMORY INITIALLY: " << memory.size() << " BLOCK LEFT");
     for (Memory::Iterator it = memory.begin(); it != memory.end(); ) {
-        if (isReadOnly(it.key())) {
-            MEMORY_DEBUG("KEEPING READ-ONLY RANGE" << it.key());
-            ++it;
-        } else {
-            it = memory.erase(it);
-        }
+        MEMORY_DEBUG("EXAMINING " << it.key());
+        it = memory.erase(it);
     }
+    MEMORY_DEBUG("RESET SNAPSHOT MEMORY FINALLY: " << memory.size() << " BLOCK LEFT");
 
     const int threadCount = threadInfo.size();
     for (int i =0; i < threadCount; i++) {
@@ -397,7 +395,7 @@ QByteArray Snapshot::gdbQsThreadInfo() const
     for (int i = 0; i < count; i++) {
         if (i)
             response += ',';
-        response += trk::hexNumber(threadInfo.at(i).id);
+        response += Coda::hexNumber(threadInfo.at(i).id);
     }
     return response;
 }
@@ -417,18 +415,18 @@ QByteArray Snapshot::gdbQThreadExtraInfo(const QByteArray &cmd) const
 
 static void gdbAppendRegister(QByteArray *ba, uint regno, uint value)
 {
-    ba->append(trk::hexNumber(regno, 2));
+    ba->append(Coda::hexNumber(regno, 2));
     ba->append(':');
-    ba->append(trk::hexNumber(trk::swapEndian(value), 8));
+    ba->append(Coda::hexNumber(Coda::swapEndian(value), 8));
     ba->append(';');
 }
 
 QByteArray Snapshot::gdbStopMessage(uint threadId, int signalNumber, bool reportThreadId) const
 {
-    QByteArray ba = ('T' + trk::hexNumber(signalNumber, 2));
+    QByteArray ba = ('T' + Coda::hexNumber(signalNumber, 2));
     if (reportThreadId) {
         ba += "thread:";
-        ba += trk::hexNumber(threadId, 3);
+        ba += Coda::hexNumber(threadId, 3);
         ba += ';';
     }
     const int threadIndex = indexOfThread(threadId);
@@ -439,7 +437,7 @@ QByteArray Snapshot::gdbStopMessage(uint threadId, int signalNumber, bool report
     // FIXME: those are not understood by gdb 6.4
     //for (int i = 16; i < 25; ++i)
     //    appendRegister(&ba, i, 0x0);
-    gdbAppendRegister(&ba, RegisterPSGdb, thread.registers[RegisterPSTrk]);
+    gdbAppendRegister(&ba, RegisterPSGdb, thread.registers[RegisterPSCoda]);
     return ba;
 }
 
@@ -449,12 +447,12 @@ QByteArray Snapshot::memoryReadLogMessage(uint addr, uint threadId, bool verbose
     QByteArray logMsg = "memory contents";
     const uint *regs = registers(threadId);
     if (verbose && regs) {
-        logMsg += " addr: " + trk::hexxNumber(addr);
+        logMsg += " addr: " + Coda::hexxNumber(addr);
         // indicate dereferencing of registers
         if (ba.size() == 4) {
             if (addr == regs[RegisterPC]) {
                 logMsg += "[PC]";
-            } else if (addr == regs[RegisterPSTrk]) {
+            } else if (addr == regs[RegisterPSCoda]) {
                 logMsg += "[PSTrk]";
             } else if (addr == regs[RegisterSP]) {
                 logMsg += "[SP]";
@@ -470,7 +468,7 @@ QByteArray Snapshot::memoryReadLogMessage(uint addr, uint threadId, bool verbose
         logMsg += " length ";
         logMsg += QByteArray::number(ba.size());
         logMsg += " :";
-        logMsg += trk::stringFromArray(ba, 16).toAscii();
+        logMsg += Coda::stringFromArray(ba, ba.size()).toAscii();
     }
     return logMsg;
 }
@@ -488,9 +486,9 @@ void Snapshot::syncRegisters(uint threadId, RegisterHandler *handler) const
         qDebug() << "HAVE: " << debuggerRegisters.size(); return);
 
     for (int i = 0; i < RegisterCount; ++i) {
-        const int gdbIndex = i == RegisterPSTrk ? int(RegisterPSGdb) : i;
+        const int gdbIndex = i == RegisterPSCoda ? int(RegisterPSGdb) : i;
         Register &reg = debuggerRegisters[gdbIndex];
-        reg.value = trk::hexxNumber(thread.registers[i]);
+        reg.value = Coda::hexxNumber(thread.registers[i]);
     }
     handler->setAndMarkRegisters(debuggerRegisters);
 }
@@ -508,7 +506,7 @@ void Snapshot::syncThreads(ThreadsHandler *handler) const
     // Take advantage of direct access to cached register values.
     Threads threads;
     const unsigned count = threadInfo.size();
-    for (unsigned t = 0; t < count; t++) {
+    for (unsigned t = 0; t < count; ++t) {
         ThreadData thread(t + 1); // Fake gdb thread ids starting from 1
             thread.targetId = QString::number(threadInfo.at(t).id);
             thread.state = threadInfo.at(t).state;
@@ -550,6 +548,83 @@ QVector<QByteArray> gdbStartupSequence()
     return s;
 }
 
+// Local symbol file handling
+
+enum { symDebug = 0 };
+
+// Build complete file name of a local sym file from DLL
+// 'QtCore.dll' to 'c:\\foo\QtCore.dll.sym'.
+
+static inline QString symFileName(const QString &folder,
+                                  const QString &libName)
+{
+    QString fileName = folder;
+    fileName.append(QLatin1Char('/'));
+    fileName.append(libName);
+    fileName.append(QLatin1String(".sym"));
+    return fileName;
+}
+
+// Look up in local symbol file matching remote library loaded in
+// cache pointed to by environmentname or in standard location
+// (next to application.sym file).
+QString localSymFileForLibrary(const QByteArray &libName,
+                               // urel/udeb: exe directory
+                               const QString &standardSymDirectory)
+{
+    // Check
+    const QByteArray envSymFileCacheDirectory = qgetenv("QTC_SYMBIAN_SYMBOLFILE_CACHE");
+    if (envSymFileCacheDirectory.isEmpty() && standardSymDirectory.isEmpty())
+        return QString();
+    // Base name
+    int lastSlashPos = libName.lastIndexOf('/');
+    if (lastSlashPos == -1)
+        lastSlashPos = libName.lastIndexOf('\\');
+    const QString libBaseName = QString::fromLatin1(lastSlashPos != - 1 ? libName.mid(lastSlashPos + 1) : libName);
+    // Check environment variable
+    if (!envSymFileCacheDirectory.isEmpty()) {
+        const QFileInfo envFi(symFileName(QString::fromLatin1(envSymFileCacheDirectory), libBaseName));
+        if (symDebug)
+            qDebug("SYM-ENV: %s exists %d\n", qPrintable(envFi.absoluteFilePath()), envFi.isFile());
+        if (envFi.isFile())
+            return envFi.absoluteFilePath();
+    }
+    // Check standard location
+    if (!standardSymDirectory.isEmpty()) {
+        const QFileInfo standardFi(symFileName(standardSymDirectory, libBaseName));
+        if (symDebug)
+            qDebug("SYM-STANDARD: %s exists %d\n", qPrintable(standardFi.absoluteFilePath()), standardFi.isFile());
+        if (standardFi.isFile())
+            return standardFi.absoluteFilePath();
+    }
+    return QString();
+}
+
+// Return a load command for a local symbol file for a library with address.
+QByteArray symFileLoadCommand(const QString &symFileNameIn,
+                              quint64 code, quint64 data)
+{
+    QByteArray symFileName = symFileNameIn.toLatin1();
+    symFileName.replace('\\', '/'); // gdb wants forward slashes
+    QByteArray command = "add-symbol-file \"";
+    command += symFileName;
+    command += "\" 0x";
+    command += QByteArray::number(code, 16);
+    if (data) {
+        command += " -s .data 0x";
+        command += QByteArray::number(data, 16);
+    }
+    return command;
+}
+
+QString msgLoadLocalSymFile(const QString &symFileName,
+                            const QByteArray &libName, quint64 code)
+{
+    return QString::fromAscii("Loading symbol file '%1' for '%2' at 0x%3").
+            arg(symFileName, QString::fromLatin1(libName)).
+            arg(code, 0, 16);
+}
+
 } // namespace Symbian
 
 // Generic gdb server helpers: Read address/length off a memory
@@ -583,7 +658,7 @@ QPair<uint, uint> parseGdbWriteRegisterWriteRequest(const QByteArray &cmd)
     const QByteArray valueName = cmd.mid(pos + 1);
     bool ok = false;
     const uint registerNumber = regName.toUInt(&ok, 16);
-    const uint value = trk::swapEndian(valueName.toUInt(&ok, 16));
+    const uint value = Coda::swapEndian(valueName.toUInt(&ok, 16));
     return QPair<uint, uint>(registerNumber, value);
 }
 
