@@ -2,28 +2,31 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
@@ -46,26 +49,32 @@
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/helpmanager.h>
-#include <coreplugin/uniqueidmanager.h>
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/command.h>
+#include <coreplugin/id.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/minisplitter.h>
+#include <coreplugin/mimedatabase.h>
 #include <coreplugin/outputpane.h>
 #include <texteditor/texteditorsettings.h>
 #include <extensionsystem/pluginmanager.h>
+#include <cpptools/cpptoolsconstants.h>
 #include <utils/qtcassert.h>
 
 #include <QtDesigner/QDesignerFormEditorPluginInterface>
-#include "qt_private/pluginmanager_p.h"
-
-#include "qt_private/iconloader_p.h"  // createIconSet
-#include "qt_private/qdesigner_formwindowmanager_p.h"
-#include "qt_private/formwindowbase_p.h"
 #include <QtDesigner/QDesignerFormEditorInterface>
 #include <QtDesigner/QDesignerComponents>
 
+#if QT_VERSION >= 0x050000
+#    include <QtDesigner/QDesignerFormWindowManagerInterface>
+#else
+#    include "qt_private/pluginmanager_p.h"
+#    include "qt_private/iconloader_p.h"  // createIconSet
+#    include "qt_private/qdesigner_formwindowmanager_p.h"
+#    include "qt_private/formwindowbase_p.h"
+#endif
+
 #include <QtDesigner/QDesignerWidgetBoxInterface>
 #include <QtDesigner/abstractobjectinspector.h>
 #include <QtDesigner/QDesignerPropertyEditorInterface>
@@ -112,7 +121,11 @@ static const char settingsGroup[] = "Designer";
 
 static inline QIcon designerIcon(const QString &iconName)
 {
+#if QT_VERSION >= 0x050000
+    const QIcon icon = QDesignerFormEditorInterface::createIcon(iconName);
+#else
     const QIcon icon = qdesigner_internal::createIconSet(iconName);
+#endif
     if (icon.isNull())
         qWarning() << "Unable to locate " << iconName;
     return icon;
@@ -121,7 +134,7 @@ static inline QIcon designerIcon(const QString &iconName)
 // Create a menu separator
 static inline QAction *createSeparator(QObject *parent,
                                  Core::ActionManager *am,
-                                 const QList<int> &context,
+                                 const Core::Context &context,
                                  Core::ActionContainer *container,
                                  const QString &name = QString(),
                                  const QString &group = QString())
@@ -154,6 +167,7 @@ FormEditorW::FormEditorW() :
     m_actionGroupPreviewInStyle(0),
     m_previewInStyleMenu(0),
     m_actionAboutPlugins(0),
+    m_actionSwitchSource(0),
     m_shortcutMapper(new QSignalMapper(this)),
     m_context(0),
     m_modeWidget(0),
@@ -174,17 +188,19 @@ FormEditorW::FormEditorW() :
     m_formeditor->setTopLevel(qobject_cast<QWidget *>(m_core->editorManager()));
     m_formeditor->setSettingsManager(new SettingsManager());
 
+#if QT_VERSION >= 0x050000
+    m_fwm = m_formeditor->formWindowManager();
+#else
     m_fwm = qobject_cast<qdesigner_internal::QDesignerFormWindowManager*>(m_formeditor->formWindowManager());
+#endif
     QTC_ASSERT(m_fwm, return);
 
-    Core::UniqueIDManager *idMan = Core::UniqueIDManager::instance();
-    m_contexts << idMan->uniqueIdentifier(QLatin1String(Designer::Constants::C_FORMEDITOR));
+    m_contexts.add(Designer::Constants::C_FORMEDITOR);
 
     setupActions();
 
     foreach (QDesignerOptionsPageInterface *designerPage, m_formeditor->optionsPages()) {
         SettingsPage *settingsPage = new SettingsPage(designerPage);
-        ExtensionSystem::PluginManager::instance()->addObject(settingsPage);
         m_settingsPages.append(settingsPage);
     }
 
@@ -211,10 +227,8 @@ FormEditorW::~FormEditorW()
     }
 
     delete m_formeditor;
-    foreach (SettingsPage *settingsPage, m_settingsPages) {
-        ExtensionSystem::PluginManager::instance()->removeObject(settingsPage);
-        delete settingsPage;
-    }
+    qDeleteAll(m_settingsPages);
+    m_settingsPages.clear();
     delete m_integration;
 
     m_self = 0;
@@ -223,7 +237,7 @@ FormEditorW::~FormEditorW()
 // Add an actioon to toggle the view state of a dock window
 void FormEditorW::addDockViewAction(Core::ActionManager *am,
                                     Core::ActionContainer *viewMenu,
-                                    int index, const QList<int> &context,
+                                    int index, const Core::Context &context,
                                     const QString &title, const QString &id)
 {
     if (const QDockWidget *dw = m_editorWidget->designerDockWidgets()[index]) {
@@ -238,7 +252,7 @@ void FormEditorW::setupViewActions()
 {
     // Populate "View" menu of form editor menu
     Core::ActionManager *am = m_core->actionManager();
-    Core::ActionContainer *viewMenu = am->actionContainer(QLatin1String(Core::Constants::M_WINDOW_VIEWS));
+    Core::ActionContainer *viewMenu = am->actionContainer(Core::Id(Core::Constants::M_WINDOW_VIEWS));
     QTC_ASSERT(viewMenu, return)
 
     addDockViewAction(am, viewMenu, WidgetBoxSubWindow, m_contexts,
@@ -293,7 +307,11 @@ void FormEditorW::fullInit()
      * This will initialize our TabOrder, Signals and slots and Buddy editors.
      */
     QList<QObject*> plugins = QPluginLoader::staticInstances();
+#if QT_VERSION >= 0x050000
+    plugins += m_formeditor->pluginInstances();
+#else
     plugins += m_formeditor->pluginManager()->instances();
+#endif
     foreach (QObject *plugin, plugins) {
         if (QDesignerFormEditorPluginInterface *formEditorPlugin = qobject_cast<QDesignerFormEditorPluginInterface*>(plugin)) {
             if (!formEditorPlugin->isInitialized())
@@ -335,14 +353,15 @@ void FormEditorW::fullInit()
     // 'Run' in 'Design' mode emits output.
     Core::MiniSplitter *splitter = new Core::MiniSplitter(Qt::Vertical);
     splitter->addWidget(m_editorWidget);
-    splitter->addWidget(new Core::OutputPanePlaceHolder(m_designMode, splitter));
+    QWidget *outputPane = new Core::OutputPanePlaceHolder(m_designMode, splitter);
+    outputPane->setObjectName(QLatin1String("DesignerOutputPanePlaceHolder"));
+    splitter->addWidget(outputPane);
     layout->addWidget(splitter);
     m_modeWidget->setLayout(layout);
 
-    Core::UniqueIDManager *idMan = Core::UniqueIDManager::instance();
-    int editorManagerContext = idMan->uniqueIdentifier(QLatin1String(Core::Constants::C_EDITORMANAGER));
-
-    m_context = new DesignerContext(QList<int>() << m_contexts << editorManagerContext, m_modeWidget, this);
+    Core::Context designerContexts = m_contexts;
+    designerContexts.add(Core::Constants::C_EDITORMANAGER);
+    m_context = new DesignerContext(designerContexts, m_modeWidget, this);
     m_core->addContextObject(m_context);
 
     m_designMode->registerDesignWidget(m_modeWidget, QStringList(QLatin1String(FORM_MIMETYPE)), m_contexts);
@@ -386,6 +405,11 @@ void FormEditorW::initDesignerSubWindows()
     m_designerSubWindows[ActionEditorSubWindow] = ae;
 }
 
+QList<Core::IOptionsPage *> FormEditorW::optionsPages() const
+{
+    return m_settingsPages;
+}
+
 void FormEditorW::ensureInitStage(InitializationStage s)
 {
     if (Designer::Constants::Internal::debug)
@@ -441,7 +465,7 @@ void FormEditorW::setupActions()
     //'delete' action. Do not set a shortcut as Designer handles
     // the 'Delete' key by event filter. Setting a shortcut triggers
     // buggy behaviour on Mac (Pressing Delete in QLineEdit removing the widget).
-    command = am->registerAction(m_fwm->actionDelete(), QLatin1String("FormEditor.Edit.Delete"), m_contexts);
+    command = am->registerAction(m_fwm->actionDelete(), Core::Id("FormEditor.Edit.Delete"), m_contexts);
     bindShortcut(command, m_fwm->actionDelete());
     command->setAttribute(Core::Command::CA_Hide);
     medit->addAction(command, Core::Constants::G_EDIT_COPYPASTE);
@@ -452,7 +476,7 @@ void FormEditorW::setupActions()
 
     m_modeActionSeparator = new QAction(this);
     m_modeActionSeparator->setSeparator(true);
-    command = am->registerAction(m_modeActionSeparator, QLatin1String("FormEditor.Sep.ModeActions"), m_contexts);
+    command = am->registerAction(m_modeActionSeparator, Core::Id("FormEditor.Sep.ModeActions"), m_contexts);
     medit->addAction(command, Core::Constants::G_EDIT_OTHER);
 
     m_toolActionIds.push_back(QLatin1String("FormEditor.WidgetEditor"));
@@ -527,13 +551,22 @@ void FormEditorW::setupActions()
     // Commands that do not go into the editor toolbar
     createSeparator(this, am, m_contexts, mformtools, QLatin1String("FormEditor.Menu.Tools.Separator2"));
 
+#if QT_VERSION >= 0x050000
+    m_actionPreview = m_fwm->action(QDesignerFormWindowManagerInterface::DefaultPreviewAction);
+#else
     m_actionPreview = m_fwm->actionDefaultPreview();
+#endif
     QTC_ASSERT(m_actionPreview, return);
     addToolAction(m_actionPreview,  am,  m_contexts,
-                   QLatin1String("FormEditor.Preview"), mformtools, tr("Ctrl+Alt+R"));
+                   QLatin1String("FormEditor.Preview"), mformtools, tr("Alt+Shift+R"));
 
     // Preview in style...
+#if QT_VERSION >= 0x050000
+    m_actionGroupPreviewInStyle = m_fwm->actionGroup(QDesignerFormWindowManagerInterface::StyledPreviewActionGroup);
+#else
     m_actionGroupPreviewInStyle = m_fwm->actionGroupPreviewInStyle();
+#endif
+
     Core::ActionContainer *previewAC = createPreviewStyleMenu(am, m_actionGroupPreviewInStyle);
     m_previewInStyleMenu = previewAC->menu();
     mformtools->addMenu(previewAC);
@@ -543,14 +576,35 @@ void FormEditorW::setupActions()
     createSeparator(this, am, m_contexts,  medit, QLatin1String("FormEditor.Edit.Separator2"), Core::Constants::G_EDIT_OTHER);
 
     createSeparator(this, am, m_contexts, mformtools, QLatin1String("FormEditor.Menu.Tools.Separator3"));
-    QAction *actionFormSettings = m_fwm->actionShowFormWindowSettingsDialog();
-    addToolAction(actionFormSettings, am, m_contexts, QLatin1String("FormEditor.FormSettings"), mformtools);
+
+    m_actionSwitchSource = new QAction(tr("Switch Source/Form"), this);
+    connect(m_actionSwitchSource, SIGNAL(triggered()), this, SLOT(switchSourceForm()));
+
+    // Switch form/source in editor/design contexts.
+    Core::Context switchContexts = m_contexts;
+    switchContexts.add(Core::Constants::C_EDITORMANAGER);
+    addToolAction(m_actionSwitchSource, am, switchContexts, QLatin1String("FormEditor.FormSwitchSource"), mformtools,
+                  tr("Shift+F4"));
 
     createSeparator(this, am, m_contexts, mformtools, QLatin1String("FormEditor.Menu.Tools.Separator4"));
+#if QT_VERSION >= 0x050000
+    QAction *actionFormSettings = m_fwm->action(QDesignerFormWindowManagerInterface::FormWindowSettingsDialogAction);
+#else
+    QAction *actionFormSettings = m_fwm->actionShowFormWindowSettingsDialog();
+#endif
+    addToolAction(actionFormSettings, am, m_contexts, QLatin1String("FormEditor.FormSettings"), mformtools);
+
+    createSeparator(this, am, m_contexts, mformtools, QLatin1String("FormEditor.Menu.Tools.Separator5"));
     m_actionAboutPlugins = new QAction(tr("About Qt Designer plugins...."), this);
     addToolAction(m_actionAboutPlugins,  am,  m_contexts,
                    QLatin1String("FormEditor.AboutPlugins"), mformtools);
-    connect(m_actionAboutPlugins,  SIGNAL(triggered()), m_fwm, SLOT(aboutPlugins()));
+    connect(m_actionAboutPlugins,  SIGNAL(triggered()), m_fwm,
+#if QT_VERSION >= 0x050000
+            SLOT(showPluginDialog())
+#else
+            SLOT(aboutPlugins())
+#endif
+            );
     m_actionAboutPlugins->setEnabled(false);
 
     // FWM
@@ -605,7 +659,7 @@ Core::ActionContainer *FormEditorW::createPreviewStyleMenu(Core::ActionManager *
         bindShortcut(command, a);
         if (isDeviceProfile) {
             command->setAttribute(Core::Command::CA_UpdateText);
-            command->setAttribute(Core::Command::CA_NonConfigureable);
+            command->setAttribute(Core::Command::CA_NonConfigurable);
         }
         menuPreviewStyle->addAction(command);
     }
@@ -642,7 +696,7 @@ void FormEditorW::bindShortcut(Core::Command *command, QAction *action)
 
 // Create an action to activate a designer tool
 QAction *FormEditorW::createEditModeAction(QActionGroup *ag,
-                                     const QList<int> &context,
+                                     const Core::Context &context,
                                      Core::ActionManager *am,
                                      Core::ActionContainer *medit,
                                      const QString &actionName,
@@ -668,7 +722,7 @@ QAction *FormEditorW::createEditModeAction(QActionGroup *ag,
 
 // Create a tool action
 Core::Command *FormEditorW::addToolAction(QAction *a, Core::ActionManager *am,
-                                          const QList<int> &context, const QString &name,
+                                          const Core::Context &context, const QString &name,
                                           Core::ActionContainer *c1, const QString &keySequence)
 {
     Core::Command *command = am->registerAction(a, name, context);
@@ -687,12 +741,18 @@ EditorData FormEditorW::createEditor(QWidget *parent)
     // Create and associate form and text editor.
     EditorData data;
     m_fwm->closeAllPreviews();
+#if QT_VERSION >= 0x050000
+    QDesignerFormWindowInterface *form = m_fwm->createFormWindow(0);
+#else
     qdesigner_internal::FormWindowBase *form = qobject_cast<qdesigner_internal::FormWindowBase *>(m_fwm->createFormWindow(0));
+#endif
     QTC_ASSERT(form, return data);
     connect(form, SIGNAL(toolChanged(int)), this, SLOT(toolChanged(int)));
     ResourceHandler *resourceHandler = new ResourceHandler(form);
+#if QT_VERSION < 0x050000
     form->setDesignerGrid(qdesigner_internal::FormWindowBase::defaultDesignerGrid());
     qdesigner_internal::FormWindowBase::setupDefaultAction(form);
+#endif
     data.widgetHost = new SharedTools::WidgetHost( /* parent */ 0, form);
     DesignerXmlEditor *xmlEditor = new DesignerXmlEditor(form, parent);
     TextEditor::TextEditorSettings::instance()->initializeEditor(xmlEditor);
@@ -703,7 +763,7 @@ EditorData FormEditorW::createEditor(QWidget *parent)
             resourceHandler, SLOT(updateResources()));
     m_editorWidget->add(data);
 
-    m_toolBar->addEditor(xmlEditor->editableInterface());
+    m_toolBar->addEditor(xmlEditor->editor());
 
     return data;
 }
@@ -796,10 +856,13 @@ void FormEditorW::print()
     const QPrinter::Orientation oldOrientation =  m_core->printer()->orientation ();
     m_core->printer()->setFullPage(false);
     do {
-
         // Grab the image to be able to a suggest suitable orientation
         QString errorMessage;
+#if QT_VERSION >= 0x050000
+        const QPixmap pixmap = m_fwm->createPreviewPixmap();
+#else
         const QPixmap pixmap = m_fwm->createPreviewPixmap(&errorMessage);
+#endif
         if (pixmap.isNull()) {
             critical(tr("The image could not be created: %1").arg(errorMessage));
             break;
@@ -840,5 +903,60 @@ void FormEditorW::print()
     m_core->printer()->setOrientation(oldOrientation);
 }
 
+// Find out current existing editor file
+static QString currentFile(const Core::EditorManager *em)
+{
+    if (Core::IEditor *editor = em->currentEditor())
+        if (const Core::IFile *file = editor->file()) {
+            const QString fileName = file->fileName();
+            if (!fileName.isEmpty() && QFileInfo(fileName).isFile())
+                return fileName;
+        }
+    return QString();
+}
+
+// Switch between form ('ui') and source file ('cpp'):
+// Find corresponding 'other' file, simply assuming it is in the same directory.
+static QString otherFile(const Core::EditorManager *em)
+{
+    // Determine mime type of current file.
+    const QString current = currentFile(em);
+    if (current.isEmpty())
+        return QString();
+    const Core::MimeDatabase *mdb = Core::ICore::instance()->mimeDatabase();
+    const Core::MimeType currentMimeType = mdb->findByFile(current);
+    if (!currentMimeType)
+        return QString();
+    // Determine potential suffixes of candidate files
+    // 'ui' -> 'cpp', 'cpp/h' -> 'ui'.
+    QStringList candidateSuffixes;
+    if (currentMimeType.type() == QLatin1String(FORM_MIMETYPE)) {
+        candidateSuffixes += mdb->findByType(QLatin1String(CppTools::Constants::CPP_SOURCE_MIMETYPE)).suffixes();
+    } else if (currentMimeType.type() == QLatin1String(CppTools::Constants::CPP_SOURCE_MIMETYPE)
+               || currentMimeType.type() == QLatin1String(CppTools::Constants::CPP_HEADER_MIMETYPE)) {
+        candidateSuffixes += mdb->findByType(QLatin1String(FORM_MIMETYPE)).suffixes();
+    } else {
+        return QString();
+    }
+    // Try to find existing file with desired suffix
+    const QFileInfo currentFI(current);
+    const QString currentBaseName = currentFI.path() + QLatin1Char('/')
+            + currentFI.baseName() + QLatin1Char('.');
+    foreach (const QString &candidateSuffix, candidateSuffixes) {
+        const QFileInfo fi(currentBaseName + candidateSuffix);
+        if (fi.isFile())
+            return fi.absoluteFilePath();
+    }
+    return QString();
+}
+
+void FormEditorW::switchSourceForm()
+{
+    Core::EditorManager *em = Core::EditorManager::instance();
+    const QString fileToOpen = otherFile(em);
+    if (!fileToOpen.isEmpty())
+        em->openEditor(fileToOpen, QString(), Core::EditorManager::ModeSwitch);
+}
+
 } // namespace Internal
 } // namespace Designer
