@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -155,6 +154,20 @@ CustomWizardFile::CustomWizardFile() :
 {
 }
 
+/*!
+    \class ProjectExplorer::CustomWizardValidationRule
+    \brief A custom wizard validation rule based on JavaScript-expressions over
+    the field placeholders.
+
+    Placeholder replacement is performed on the condition and it is evaluated
+    using JavaScript. So, for example '"%ProjectName%" != "untitled" would block
+    default names. On failure, the message is displayed in a red warning label
+    in the wizard page. Placeholder replacement is also performed on the message
+    prior to displaying.
+
+    \sa ProjectExplorer::CustomWizard, ProjectExplorer::CustomWizardFieldPage
+*/
+
 bool CustomWizardValidationRule::validateRules(const QList<CustomWizardValidationRule> &rules,
                                                const QMap<QString, QString> &replacementMap,
                                                QString *errorMessage)
@@ -392,7 +405,7 @@ static ParseState nextOpeningState(ParseState in, const QStringRef &name)
         break;
     }
     return ParseError;
-};
+}
 
 // Switch parser state depending on closing element name.
 static ParseState nextClosingState(ParseState in, const QStringRef &name)
@@ -458,7 +471,7 @@ static ParseState nextClosingState(ParseState in, const QStringRef &name)
         break;
     }
     return ParseError;
-};
+}
 
 // Parse kind attribute
 static inline Core::IWizard::WizardKind kindAttribute(const QXmlStreamReader &r)
@@ -511,10 +524,9 @@ static inline QString attributeValue(const QXmlStreamReader &r, const char *name
 }
 
 // Return locale language attribute "de_UTF8" -> "de", empty string for "C"
-static inline QString localeLanguage()
+static inline QString languageSetting()
 {
-    QLocale loc;
-    QString name = loc.name();
+    QString name = Core::ICore::instance()->userInterfaceLanguage();
     const int underScorePos = name.indexOf(QLatin1Char('_'));
     if (underScorePos != -1)
         name.truncate(underScorePos);
@@ -523,6 +535,16 @@ static inline QString localeLanguage()
     return name;
 }
 
+/*!
+    \class ProjectExplorer::Internal::GeneratorScriptArgument
+    \brief Argument to a custom wizard generator script.
+
+    Contains placeholders to be replaced by field values or file names
+    as in \c '--class-name=%ClassName%' or \c '--description=%Description%'.
+
+    \sa ProjectExplorer::CustomWizard
+*/
+
 GeneratorScriptArgument::GeneratorScriptArgument(const QString &v) :
     value(v), flags(0)
 {
@@ -542,7 +564,7 @@ CustomWizardParameters::ParseResult
     clear();
     bp->clear();
     bp->setKind(Core::IWizard::ProjectWizard);
-    const QString language = localeLanguage();
+    const QString language = languageSetting();
     CustomWizardField field;
     do {
         token = reader.readNext();
@@ -749,6 +771,38 @@ QString CustomWizardParameters::toString() const
 
 static inline QString passThrough(const QString &in) { return in; }
 
+static inline QString headerGuard(const QString &in)
+{
+    QString result;
+    foreach (const QChar c, in) {
+        if (c.isLetterOrNumber())
+            result.append(c.toUpper());
+        else
+            result.append(QLatin1Char('_'));
+    }
+    return result;
+}
+
+static inline QString structName(const QString &in)
+{
+    bool capNeeded = true;
+    QString result;
+    foreach (const QChar c, in) {
+        if (c.isLetterOrNumber()) {
+            if (capNeeded) {
+                result.append(c.toUpper());
+                capNeeded = false;
+            } else {
+                result.append(c);
+            }
+        } else {
+            result.append(QLatin1Char('_'));
+            capNeeded = true;
+        }
+    }
+    return result;
+}
+
 // Do field replacements applying modifiers and string transformation
 // for the value
 template <class ValueStringTransformation>
@@ -799,7 +853,13 @@ bool replaceFieldHelper(ValueStringTransformation transform,
         case 'u':
             replacement = it.value().toUpper();
             break;
-        case 'c': // Capitalize first letter
+        case 'h': // Create a header guard.
+            replacement = headerGuard(it.value());
+            break;
+        case 's': // Create a struct or class name.
+            replacement = structName(it.value());
+            break;
+        case 'c': // Capitalize first letter.
             replacement = it.value();
             if (!replacement.isEmpty())
                 replacement[0] = replacement.at(0).toUpper();
@@ -817,6 +877,22 @@ bool replaceFieldHelper(ValueStringTransformation transform,
     return nonEmptyReplacements;
 }
 
+/*!
+    \brief Performs field replacements.
+
+    Replace field values delimited by '%' with special modifiers:
+    \list
+    \o %Field% -> simple replacement
+    \o %Field:l% -> replace with everything changed to lower case
+    \o %Field:u% -> replace with everything changed to upper case
+    \o %Field:c% -> replace with first character capitalized
+    \o  %Field:h% -> replace with something usable as header guard
+    \o %Field:s% -> replace with something usable as structure or class name
+    \endlist
+
+    The return value indicates whether non-empty replacements were encountered.
+*/
+
 bool CustomWizardContext::replaceFields(const FieldReplacementMap &fm, QString *s)
 {
     return replaceFieldHelper(passThrough, fm, s);
@@ -861,6 +937,29 @@ QString TemporaryFileTransform::operator()(const QString &value) const
     return name;
 }
 
+/*!
+    \class ProjectExplorer::Internal::CustomWizardContext
+    \brief Context used for one custom wizard run.
+
+    Shared between CustomWizard and the CustomWizardPage as it is used
+    for the QLineEdit-type fields'
+    default texts as well. Contains basic replacement fields
+    like  \c '%CppSourceSuffix%', \c '%CppHeaderSuffix%' (settings-dependent)
+    reset() should be called before each wizard run to refresh them.
+    CustomProjectWizard additionally inserts \c '%ProjectName%' from
+    the intro page to have it available for default texts.
+
+    \sa ProjectExplorer::CustomWizard
+*/
+
+/*!
+    \brief Special replaceFields() overload used for the arguments of a generator
+    script.
+
+    Write the expanded field values out to temporary files and
+    inserts file names instead of the expanded fields in string 's'.
+*/
+
 bool CustomWizardContext::replaceFields(const FieldReplacementMap &fm, QString *s,
                                         TemporaryFilePtrList *files)
 {
