@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -46,12 +45,13 @@
 #include <QtXml/QXmlStreamReader>
 #include <QtCore/QDebug>
 #include <QPlainTextEdit>
+#include <QHashIterator>
 
 #include "abstractview.h"
 #include "nodeinstanceview.h"
 #include "metainfo.h"
+#include "nodemetainfo.h"
 #include "model_p.h"
-#include "modelutilities.h"
 #include "subcomponentmanager.h"
 #include "variantparser.h"
 #include "internalproperty.h"
@@ -92,9 +92,11 @@ namespace Internal {
 
 ModelPrivate::ModelPrivate(Model *model) :
         m_q(model),
-        m_writeLock(false)
+        m_writeLock(false),
+        m_internalIdCounter(1)
 {
-    m_rootInternalNode = createNode("Qt/Item", 4, 7, PropertyListType());
+    m_rootInternalNode = createNode("QtQuick/Item", 1, 0, PropertyListType(), PropertyListType(), QString(), ModelNode::NodeWithoutSource,true);
+    m_acutalStateNode = m_rootInternalNode;
 }
 
 ModelPrivate::~ModelPrivate()
@@ -108,81 +110,73 @@ void ModelPrivate::detachAllViews()
         detachView(view.data(), true);
 
     m_viewList.clear();
+
+    if (m_rewriterView) {
+        m_rewriterView->modelAboutToBeDetached(m_q);
+        m_rewriterView.clear();
+    }
+
+    if (m_nodeInstanceView) {
+        m_nodeInstanceView->modelAboutToBeDetached(m_q);
+        m_nodeInstanceView.clear();
+    }
 }
 
-Model *ModelPrivate::create(QString type, int major, int minor)
+Model *ModelPrivate::create(QString type, int major, int minor, Model *metaInfoPropxyModel)
 {
     Model *model = new Model;
 
-    model->m_d->rootNode()->setType(type);
-    model->m_d->rootNode()->setMajorVersion(major);
-    model->m_d->rootNode()->setMinorVersion(minor);
+    model->d->m_metaInfoProxyModel = metaInfoPropxyModel;
+    model->d->rootNode()->setType(type);
+    model->d->rootNode()->setMajorVersion(major);
+    model->d->rootNode()->setMinorVersion(minor);
 
     return model;
 }
 
-void ModelPrivate::addImport(const Import &import)
+void ModelPrivate::changeImports(const QList<Import> &toBeAddedImportList, const QList<Import> &toBeRemovedImportList)
 {
-    if (m_imports.contains(import))
-        return;
+    QList<Import> removedImportList;
+    foreach (const Import &import, toBeRemovedImportList) {
+        if (m_imports.contains(import)) {
+            removedImportList.append(import);
+            m_imports.removeOne(import);
+        }
+    }
 
-    m_imports.append(import);
-    notifyImportAdded(import);
+    QList<Import> addedImportList;
+    foreach (const Import &import, toBeAddedImportList) {
+        if (!m_imports.contains(import)) {
+            addedImportList.append(import);
+            m_imports.append(import);
+        }
+    }
+
+    if (!removedImportList.isEmpty() || !addedImportList.isEmpty())
+        notifyImportsChanged(addedImportList, removedImportList);
 }
 
-void ModelPrivate::removeImport(const Import &import)
-{
-    if (!m_imports.removeOne(import))
-        return;
-
-    notifyImportRemoved(import);
-}
-
-void ModelPrivate::notifyImportAdded(const Import &import)
+void ModelPrivate::notifyImportsChanged(const QList<Import> &addedImports, const QList<Import> &removedImports)
 {
     bool resetModel = false;
     QString description;
 
     try {
         if (rewriterView()) {
-            rewriterView()->importAdded(import);
+            rewriterView()->importsChanged(addedImports, removedImports);
         }
     } catch (RewritingException &e) {
         description = e.description();
         resetModel = true;
     }
 
-    if (rewriterView())
-    foreach (const QWeakPointer<AbstractView> &view, m_viewList)
-        view->importAdded(import);
+    NodeMetaInfo::clearCache();
 
     if (nodeInstanceView())
-        nodeInstanceView()->importAdded(import);
-
-    if (resetModel) {
-        resetModelByRewriter(description);
-    }
-}
-
-void ModelPrivate::notifyImportRemoved(const Import &import)
-{
-    bool resetModel = false;
-    QString description;
-
-    try {
-        if (rewriterView()) {
-            rewriterView()->importRemoved(import);
-        }
-    } catch (RewritingException &e) {
-        description = e.description();
-        resetModel = true;
-    }
+        nodeInstanceView()->importsChanged(addedImports, removedImports);
 
     foreach (const QWeakPointer<AbstractView> &view, m_viewList)
-        view->importRemoved(import);
-
-    if (nodeInstanceView())
-        nodeInstanceView()->importRemoved(import);
+        view->importsChanged(addedImports, removedImports);
 
     if (resetModel) {
         resetModelByRewriter(description);
@@ -209,12 +203,22 @@ void ModelPrivate::setFileUrl(const QUrl &fileUrl)
 InternalNode::Pointer ModelPrivate::createNode(const QString &typeString,
                                                int majorVersion,
                                                int minorVersion,
-                                               const QList<QPair<QString, QVariant> > &propertyList)
+                                               const QList<QPair<QString, QVariant> > &propertyList,
+                                               const QList<QPair<QString, QVariant> > &auxPropertyList,
+                                               const QString &nodeSource,
+                                               ModelNode::NodeSourceType nodeSourceType,
+                                               bool isRootNode)
 {
     if (typeString.isEmpty())
         throw InvalidArgumentException(__LINE__, __FUNCTION__, __FILE__, tr("invalid type"));
 
-    InternalNode::Pointer newInternalNodePointer = InternalNode::create(typeString, majorVersion, minorVersion);
+    qint32 internalId = 0;
+
+    if (!isRootNode)
+        internalId = m_internalIdCounter++;
+
+    InternalNode::Pointer newInternalNodePointer = InternalNode::create(typeString, majorVersion, minorVersion, internalId);
+    newInternalNodePointer->setNodeSourceType(nodeSourceType);
 
     typedef QPair<QString, QVariant> PropertyPair;
 
@@ -223,7 +227,15 @@ InternalNode::Pointer ModelPrivate::createNode(const QString &typeString,
         newInternalNodePointer->variantProperty(propertyPair.first)->setValue(propertyPair.second);
     }
 
+    foreach (const PropertyPair &propertyPair, auxPropertyList) {
+        newInternalNodePointer->setAuxiliaryData(propertyPair.first, propertyPair.second);
+    }
+
     m_nodeSet.insert(newInternalNodePointer);
+    m_internalIdNodeHash.insert(newInternalNodePointer->internalId(), newInternalNodePointer);
+
+    if (!nodeSource.isNull())
+        newInternalNodePointer->setNodeSource(nodeSource);
 
     notifyNodeCreated(newInternalNodePointer);
 
@@ -240,6 +252,7 @@ void ModelPrivate::removeNodeFromModel(const InternalNodePointer &node)
         m_idNodeHash.remove(node->id());
     node->setValid(false);
     m_nodeSet.remove(node);
+    m_internalIdNodeHash.remove(node->internalId());
 }
 
 void ModelPrivate::removeAllSubNodes(const InternalNode::Pointer &node)
@@ -355,6 +368,38 @@ void ModelPrivate::notifyAuxiliaryDataChanged(const InternalNodePointer &interna
     }
 }
 
+void ModelPrivate::notifyNodeSourceChanged(const InternalNodePointer &internalNode, const QString &newNodeSource)
+{
+    bool resetModel = false;
+    QString description;
+
+    try {
+        if (rewriterView()) {
+            ModelNode node(internalNode, model(), rewriterView());
+            rewriterView()->nodeSourceChanged(node, newNodeSource);
+        }
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        Q_ASSERT(view != 0);
+        ModelNode node(internalNode, model(), view.data());
+        view->nodeSourceChanged(node, newNodeSource);
+
+    }
+
+    if (nodeInstanceView()) {
+        ModelNode node(internalNode, model(), nodeInstanceView());
+        nodeInstanceView()->nodeSourceChanged(node, newNodeSource);
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
 void ModelPrivate::notifyRootNodeTypeChanged(const QString &type, int majorVersion, int minorVersion)
 {
     bool resetModel = false;
@@ -368,15 +413,15 @@ void ModelPrivate::notifyRootNodeTypeChanged(const QString &type, int majorVersi
         resetModel = true;
     }
 
+    if (nodeInstanceView())
+        nodeInstanceView()->rootNodeTypeChanged(type, majorVersion, minorVersion);
+
     foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
         Q_ASSERT(view != 0);
         view->rootNodeTypeChanged(type, majorVersion, minorVersion);
 
     }
 
-    if (nodeInstanceView())
-        nodeInstanceView()->rootNodeTypeChanged(type, majorVersion, minorVersion);
-
     if (resetModel) {
         resetModelByRewriter(description);
     }
@@ -400,6 +445,275 @@ void ModelPrivate::notifyInstancePropertyChange(const QList<QPair<ModelNode, QSt
     }
 }
 
+void ModelPrivate::notifyInstancesCompleted(const QVector<ModelNode> &nodeVector)
+{
+    bool resetModel = false;
+    QString description;
+
+    QVector<Internal::InternalNode::Pointer> internalVector(toInternalNodeVector(nodeVector));
+
+    try {
+        if (rewriterView())
+            rewriterView()->instancesCompleted(toModelNodeVector(internalVector, rewriterView()));
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        Q_ASSERT(view != 0);
+        view->instancesCompleted(toModelNodeVector(internalVector, view.data()));
+    }
+
+    if (nodeInstanceView()) {
+        nodeInstanceView()->instancesCompleted(toModelNodeVector(internalVector, nodeInstanceView()));
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
+QMultiHash<ModelNode, InformationName> convertModelNodeInformationHash(const QMultiHash<ModelNode, InformationName> &informationChangeHash, AbstractView *view)
+{
+    QMultiHash<ModelNode, InformationName>  convertedModelNodeInformationHash;
+
+    QHashIterator<ModelNode, InformationName> hashIterator(informationChangeHash);
+    while (hashIterator.hasNext()) {
+        hashIterator.next();
+        convertedModelNodeInformationHash.insert(ModelNode(hashIterator.key(), view), hashIterator.value());
+    }
+
+    return convertedModelNodeInformationHash;
+}
+
+void ModelPrivate::notifyInstancesInformationsChange(const QMultiHash<ModelNode, InformationName> &informationChangeHash)
+{
+    bool resetModel = false;
+    QString description;
+
+    try {
+        if (rewriterView())
+            rewriterView()->instanceInformationsChange(convertModelNodeInformationHash(informationChangeHash, rewriterView()));
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        Q_ASSERT(view != 0);
+        view->instanceInformationsChange(convertModelNodeInformationHash(informationChangeHash, view.data()));
+    }
+
+    if (nodeInstanceView()) {
+        nodeInstanceView()->instanceInformationsChange(convertModelNodeInformationHash(informationChangeHash, nodeInstanceView()));
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
+void ModelPrivate::notifyInstancesRenderImageChanged(const QVector<ModelNode> &nodeVector)
+{
+    bool resetModel = false;
+    QString description;
+
+    QVector<Internal::InternalNode::Pointer> internalVector(toInternalNodeVector(nodeVector));
+
+    try {
+        if (rewriterView())
+            rewriterView()->instancesRenderImageChanged(toModelNodeVector(internalVector, rewriterView()));
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        Q_ASSERT(view != 0);
+        view->instancesRenderImageChanged(toModelNodeVector(internalVector, view.data()));
+    }
+
+    if (nodeInstanceView()) {
+        nodeInstanceView()->instancesRenderImageChanged(toModelNodeVector(internalVector, nodeInstanceView()));
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
+void ModelPrivate::notifyInstancesPreviewImageChanged(const QVector<ModelNode> &nodeVector)
+{
+    bool resetModel = false;
+    QString description;
+
+    QVector<Internal::InternalNode::Pointer> internalVector(toInternalNodeVector(nodeVector));
+
+    try {
+        if (rewriterView())
+            rewriterView()->instancesPreviewImageChanged(toModelNodeVector(internalVector, rewriterView()));
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        Q_ASSERT(view != 0);
+        view->instancesPreviewImageChanged(toModelNodeVector(internalVector, view.data()));
+    }
+
+    if (nodeInstanceView()) {
+        nodeInstanceView()->instancesPreviewImageChanged(toModelNodeVector(internalVector, nodeInstanceView()));
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
+void ModelPrivate::notifyInstancesChildrenChanged(const QVector<ModelNode> &nodeVector)
+{
+    bool resetModel = false;
+    QString description;
+
+    QVector<Internal::InternalNode::Pointer> internalVector(toInternalNodeVector(nodeVector));
+
+    try {
+        if (rewriterView())
+            rewriterView()->instancesChildrenChanged(toModelNodeVector(internalVector, rewriterView()));
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        Q_ASSERT(view != 0);
+        view->instancesChildrenChanged(toModelNodeVector(internalVector, view.data()));
+    }
+
+    if (nodeInstanceView()) {
+        nodeInstanceView()->instancesChildrenChanged(toModelNodeVector(internalVector, nodeInstanceView()));
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
+void ModelPrivate::notifyActualStateChanged(const ModelNode &node)
+{
+    bool resetModel = false;
+    QString description;
+
+    m_acutalStateNode = node.internalNode();
+
+    try {
+        if (rewriterView())
+            rewriterView()->actualStateChanged(ModelNode(node.internalNode(), model(), rewriterView()));
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        Q_ASSERT(view != 0);
+        view->actualStateChanged(ModelNode(node.internalNode(), model(), view.data()));
+    }
+
+    if (nodeInstanceView()) {
+        nodeInstanceView()->actualStateChanged(ModelNode(node.internalNode(), model(), nodeInstanceView()));
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
+void ModelPrivate::notifyRewriterBeginTransaction()
+{
+    bool resetModel = false;
+    QString description;
+
+    try {
+        if (rewriterView())
+            rewriterView()->rewriterBeginTransaction();
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        Q_ASSERT(view != 0);
+        view->rewriterBeginTransaction();
+    }
+
+    if (nodeInstanceView()) {
+        nodeInstanceView()->rewriterBeginTransaction();
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
+void ModelPrivate::notifyRewriterEndTransaction()
+{
+    bool resetModel = false;
+    QString description;
+
+    try {
+        if (rewriterView())
+            rewriterView()->rewriterEndTransaction();
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        Q_ASSERT(view != 0);
+        view->rewriterEndTransaction();
+    }
+
+    if (nodeInstanceView()) {
+        nodeInstanceView()->rewriterEndTransaction();
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
+void ModelPrivate::notifyInstanceToken(const QString &token, int number, const QVector<ModelNode> &nodeVector)
+{
+    bool resetModel = false;
+    QString description;
+
+    QVector<Internal::InternalNode::Pointer> internalVector(toInternalNodeVector(nodeVector));
+
+
+    try {
+        if (rewriterView())
+            rewriterView()->instancesToken(token, number, toModelNodeVector(internalVector, rewriterView()));
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        Q_ASSERT(view != 0);
+        view->instancesToken(token, number, toModelNodeVector(internalVector, view.data()));
+    }
+
+    if (nodeInstanceView()) {
+        nodeInstanceView()->instancesToken(token, number, toModelNodeVector(internalVector, nodeInstanceView()));
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
 void ModelPrivate::notifyCustomNotification(const AbstractView *senderView, const QString &identifier, const QList<ModelNode> &nodeList, const QList<QVariant> &data)
 {
     bool resetModel = false;
@@ -822,6 +1136,64 @@ void ModelPrivate::notifyVariantPropertiesChanged(const InternalNodePointer &int
     }
 }
 
+void ModelPrivate::notifyNodeAboutToBeReparent(const InternalNodePointer &internalNodePointer, const InternalNodeAbstractPropertyPointer &newPropertyParent, const InternalNodePointer &oldParent, const QString &oldPropertyName, AbstractView::PropertyChangeFlags propertyChange)
+{
+    bool resetModel = false;
+    QString description;
+
+    try {
+        if (rewriterView()) {
+            NodeAbstractProperty newProperty;
+            NodeAbstractProperty oldProperty;
+
+            if (!oldPropertyName.isEmpty() && oldParent->isValid())
+                oldProperty = NodeAbstractProperty(oldPropertyName, oldParent, model(), rewriterView());
+
+            if (!newPropertyParent.isNull())
+                newProperty = NodeAbstractProperty(newPropertyParent, model(), rewriterView());
+            ModelNode node(internalNodePointer, model(), rewriterView());
+            rewriterView()->nodeAboutToBeReparented(node, newProperty, oldProperty, propertyChange);
+        }
+    } catch (RewritingException &e) {
+        description = e.description();
+        resetModel = true;
+    }
+
+    foreach (const QWeakPointer<AbstractView> &view, m_viewList) {
+        NodeAbstractProperty newProperty;
+        NodeAbstractProperty oldProperty;
+
+        Q_ASSERT(!view.isNull());
+        if (!oldPropertyName.isEmpty() && oldParent->isValid())
+            oldProperty = NodeAbstractProperty(oldPropertyName, oldParent, model(), view.data());
+
+        if (!newPropertyParent.isNull())
+            newProperty = NodeAbstractProperty(newPropertyParent, model(), view.data());
+        ModelNode node(internalNodePointer, model(), view.data());
+
+        view->nodeAboutToBeReparented(node, newProperty, oldProperty, propertyChange);
+
+    }
+
+    if (nodeInstanceView()) {
+        NodeAbstractProperty newProperty;
+        NodeAbstractProperty oldProperty;
+
+        if (!oldPropertyName.isEmpty() && oldParent->isValid())
+            oldProperty = NodeAbstractProperty(oldPropertyName, oldParent, model(), nodeInstanceView());
+
+        if (!newPropertyParent.isNull())
+            newProperty = NodeAbstractProperty(newPropertyParent, model(), nodeInstanceView());
+        ModelNode node(internalNodePointer, model(), nodeInstanceView());
+        nodeInstanceView()->nodeAboutToBeReparented(node, newProperty, oldProperty, propertyChange);
+    }
+
+    if (resetModel) {
+        resetModelByRewriter(description);
+    }
+}
+
+
 void ModelPrivate::notifyNodeReparent(const InternalNode::Pointer &internalNodePointer, const InternalNodeAbstractProperty::Pointer &newPropertyParent, const InternalNodePointer &oldParent, const QString &oldPropertyName, AbstractView::PropertyChangeFlags propertyChange)
 {
     bool resetModel = false;
@@ -954,6 +1326,15 @@ QList<ModelNode> ModelPrivate::toModelNodeList(const QList<InternalNode::Pointer
     return newNodeList;
 }
 
+QVector<ModelNode> ModelPrivate::toModelNodeVector(const QVector<InternalNode::Pointer> &nodeVector, AbstractView *view) const
+{
+    QVector<ModelNode> newNodeVector;
+    foreach (const Internal::InternalNode::Pointer &node, nodeVector)
+        newNodeVector.append(ModelNode(node, model(), view));
+
+    return newNodeVector;
+}
+
 QList<Internal::InternalNode::Pointer> ModelPrivate::toInternalNodeList(const QList<ModelNode> &nodeList) const
 {
     QList<Internal::InternalNode::Pointer> newNodeList;
@@ -963,6 +1344,15 @@ QList<Internal::InternalNode::Pointer> ModelPrivate::toInternalNodeList(const QL
     return newNodeList;
 }
 
+QVector<Internal::InternalNode::Pointer> ModelPrivate::toInternalNodeVector(const QVector<ModelNode> &nodeVector) const
+{
+    QVector<Internal::InternalNode::Pointer> newNodeVector;
+    foreach (const ModelNode &node, nodeVector)
+        newNodeVector.append(node.internalNode());
+
+    return newNodeVector;
+}
+
 void ModelPrivate::changeSelectedNodes(const QList<InternalNode::Pointer> &newSelectedNodeList,
                                        const QList<InternalNode::Pointer> &oldSelectedNodeList)
 {
@@ -1107,6 +1497,9 @@ void ModelPrivate::reparentNode(const InternalNode::Pointer &newParentNode, cons
 
     InternalNodeAbstractProperty::Pointer newParentProperty(newParentNode->nodeAbstractProperty(name));
     Q_ASSERT(!newParentProperty.isNull());
+
+    notifyNodeAboutToBeReparent(node, newParentProperty, oldParentNode, oldParentPropertyName, propertyChange);
+
     if (newParentProperty)
         node->setParentProperty(newParentProperty);
 
@@ -1151,6 +1544,12 @@ void ModelPrivate::setScriptFunctions(const InternalNode::Pointer &internalNode,
     notifyScriptFunctionsChanged(internalNode, scriptFunctionList);
 }
 
+void ModelPrivate::setNodeSource(const InternalNodePointer &internalNode, const QString &nodeSource)
+{
+    internalNode->setNodeSource(nodeSource);
+    notifyNodeSourceChanged(internalNode, nodeSource);
+}
+
 void ModelPrivate::changeNodeOrder(const InternalNode::Pointer &internalParentNode, const QString &listPropertyName, int from, int to)
 {
     InternalNodeListProperty::Pointer nodeList(internalParentNode->nodeListProperty(listPropertyName));
@@ -1161,15 +1560,6 @@ void ModelPrivate::changeNodeOrder(const InternalNode::Pointer &internalParentNo
     notifyNodeOrderChanged(nodeList, internalNode, from);
 }
 
-void ModelPrivate::setRootNode(const InternalNode::Pointer& newRootNode)
-{
-    removeNode(m_rootInternalNode);
-    m_rootInternalNode = newRootNode;
-
-    if (!m_rootInternalNode.isNull() && m_rootInternalNode->isValid())
-        notifyNodeCreated(m_rootInternalNode);
-}
-
 void  ModelPrivate::setRewriterView(RewriterView *rewriterView)
 {
     if (rewriterView == m_rewriterView.data())
@@ -1220,6 +1610,16 @@ bool ModelPrivate::hasId(const QString &id) const
     return m_idNodeHash.contains(id);
 }
 
+InternalNodePointer ModelPrivate::nodeForInternalId(qint32 internalId) const
+{
+    return m_internalIdNodeHash.value(internalId);
+}
+
+bool ModelPrivate::hasNodeForInternalId(qint32 internalId) const
+{
+    return m_internalIdNodeHash.contains(internalId);
+}
+
 QList<InternalNodePointer> ModelPrivate::allNodes() const
 {
     // the item must be ordered!
@@ -1241,6 +1641,11 @@ bool ModelPrivate::isWriteLocked() const
     return m_writeLock;
 }
 
+InternalNode::Pointer ModelPrivate::actualStateNode() const
+{
+    return m_acutalStateNode;
+}
+
 
 WriteLocker::WriteLocker(ModelPrivate *model)
     : m_model(model)
@@ -1254,9 +1659,9 @@ WriteLocker::WriteLocker(ModelPrivate *model)
 }
 
 WriteLocker::WriteLocker(Model *model)
-    : m_model(model->m_d)
+    : m_model(model->d)
 {
-    Q_ASSERT(model->m_d);
+    Q_ASSERT(model->d);
     if (m_model->m_writeLock)
         qWarning() << "QmlDesigner: Misbehaving view calls back to model!!!";
     // FIXME: Enable it again
@@ -1311,20 +1716,20 @@ WriteLocker::~WriteLocker()
 
 Model::Model()  :
    QObject(),
-   m_d(new Internal::ModelPrivate(this))
+   d(new Internal::ModelPrivate(this))
 {
 }
 
 
 Model::~Model()
 {
-    delete m_d;
+    delete d;
 }
 
 
-Model *Model::create(QString type, int major, int minor)
+Model *Model::create(QString type, int major, int minor, Model *metaInfoPropxyModel)
 {
-    return Internal::ModelPrivate::create(type, major, minor);
+    return Internal::ModelPrivate::create(type, major, minor, metaInfoPropxyModel);
 }
 
 
@@ -1350,12 +1755,12 @@ Model *Model::create(QString type, int major, int minor)
 //            = componentNode.baseNodeState().m_internalNodeState->propertyLocation("data");
 //
 //    TextModifier *textModifier
-//            = new Internal::ComponentTextModifier(m_d->m_rewriter->textModifier(),
+//            = new Internal::ComponentTextModifier(d->m_rewriter->textModifier(),
 //                                                  componentRootLocation,
-//                                                  m_d->m_rootInternalNode->baseNodeState()->location());
+//                                                  d->m_rootInternalNode->baseNodeState()->location());
 //
 //    QList<QDeclarativeError> errors;
-//    Model *subModel = create(textModifier, m_d->m_fileUrl, &errors);
+//    Model *subModel = create(textModifier, d->m_fileUrl, &errors);
 //
 //    Q_ASSERT(subModel);
 //    Q_ASSERT(errors.size() == 0); // should never happen, after all it was already compiled!
@@ -1367,17 +1772,79 @@ Model *Model::create(QString type, int major, int minor)
 
 QList<Import> Model::imports() const
 {
-    return m_d->imports();
+    return d->imports();
 }
 
-void Model::addImport(const Import &import)
+void Model::changeImports(const QList<Import> &importsToBeAdded, const QList<Import> &importsToBeRemoved)
 {
-    m_d->addImport(import);
+    d->changeImports(importsToBeAdded, importsToBeRemoved);
 }
 
-void Model::removeImport(const Import &import)
+
+static bool compareVersions(const QString &version1, const QString &version2, bool allowHigherVersion)
 {
-    m_d->removeImport(import);
+    if (version1 == version2)
+        return true;
+    if (!allowHigherVersion)
+        return false;
+    QStringList version1List = version1.split('.');
+    QStringList version2List = version2.split('.');
+    if (version1List.count() == 2 && version2List.count() == 2) {
+        bool ok;
+        int major1 = version1List.first().toInt(&ok);
+        if (!ok)
+            return false;
+        int major2 = version2List.first().toInt(&ok);
+        if (!ok)
+            return false;
+        if (major1 >= major2) {
+            int minor1 = version1List.last().toInt(&ok);
+            if (!ok)
+                return false;
+            int minor2 = version2List.last().toInt(&ok);
+            if (!ok)
+                return false;
+            if (minor1 >= minor2)
+                return true;
+        }
+    }
+
+    return false;
+}
+
+bool Model::hasImport(const Import &import, bool ignoreAlias, bool allowHigherVersion)
+{
+    if (imports().contains(import))
+        return true;
+    if (!ignoreAlias)
+        return false;
+
+    foreach (const Import &existingImport, imports()) {
+        if (existingImport.isFileImport() && import.isFileImport())
+            if (existingImport.file() == import.file() && compareVersions(existingImport.version(), import.version(), allowHigherVersion))
+                return true;
+        if (existingImport.isLibraryImport() && import.isLibraryImport())
+            if (existingImport.url() == import.url()  && compareVersions(existingImport.version(), import.version(), allowHigherVersion))
+                return true;
+    }
+    return false;
+}
+
+RewriterView *Model::rewriterView() const
+{
+    return d->rewriterView();
+}
+
+/*!
+ \brief Returns the model that is used for metainfo
+ \return Return itself if not other metaInfoProxyModel does exist
+*/
+Model *Model::metaInfoProxyModel()
+{
+    if (d->m_metaInfoProxyModel)
+        return d->m_metaInfoProxyModel->metaInfoProxyModel();
+    else
+        return this;
 }
 
 #if 0
@@ -1395,12 +1862,12 @@ Model *Model::create(const QString &rootType)
 
 Model *Model::masterModel() const
 {
-    return m_d->m_masterModel.data();
+    return d->m_masterModel.data();
 }
 
 void Model::setMasterModel(Model *model)
 {
-    m_d->m_masterModel = model;
+    d->m_masterModel = model;
 }
 
 /*!
@@ -1409,7 +1876,7 @@ void Model::setMasterModel(Model *model)
   */
 QUrl Model::fileUrl() const
 {
-    return m_d->fileUrl();
+    return d->fileUrl();
 }
 
 /*!
@@ -1418,8 +1885,8 @@ QUrl Model::fileUrl() const
   */
 void Model::setFileUrl(const QUrl &url)
 {
-    Internal::WriteLocker locker(m_d);
-    m_d->setFileUrl(url);
+    Internal::WriteLocker locker(d);
+    d->setFileUrl(url);
 }
 
 /*!
@@ -1427,16 +1894,17 @@ void Model::setFileUrl(const QUrl &url)
   */
 const MetaInfo Model::metaInfo() const
 {
-    return m_d->metaInfo();
+    return d->metaInfo();
 }
 
-/*!
-  \brief Sets a specific Metainfo on this Model
-  */
-void Model::setMetaInfo(const MetaInfo &metaInfo)
+bool Model::hasNodeMetaInfo(const QString &typeName, int majorVersion, int minorVersion)
 {
-    Internal::WriteLocker locker(m_d);
-    m_d->setMetaInfo(metaInfo);
+    return NodeMetaInfo(metaInfoProxyModel(), typeName, majorVersion, minorVersion).isValid();
+}
+
+NodeMetaInfo Model::metaInfo(const QString &typeName, int majorVersion, int minorVersion)
+{
+    return NodeMetaInfo(metaInfoProxyModel(), typeName, majorVersion, minorVersion);
 }
 
 /*!
@@ -1444,7 +1912,7 @@ void Model::setMetaInfo(const MetaInfo &metaInfo)
   */
 MetaInfo Model::metaInfo()
 {
-    return m_d->metaInfo();
+    return d->metaInfo();
 }
 
 /*! \name Undo Redo Interface
@@ -1468,20 +1936,20 @@ The view is informed that it has been registered within the model by a call to A
 */
 void Model::attachView(AbstractView *view)
 {
-//    Internal::WriteLocker locker(m_d);
+//    Internal::WriteLocker locker(d);
     RewriterView *rewriterView = qobject_cast<RewriterView*>(view);
     if (rewriterView) {
-        m_d->setRewriterView(rewriterView);
+        d->setRewriterView(rewriterView);
         return;
     }
 
     NodeInstanceView *nodeInstanceView = qobject_cast<NodeInstanceView*>(view);
     if (nodeInstanceView) {
-        m_d->setNodeInstanceView(nodeInstanceView);
+        d->setNodeInstanceView(nodeInstanceView);
         return;
     }
 
-    m_d->attachView(view);
+    d->attachView(view);
 }
 
 /*!
@@ -1494,22 +1962,22 @@ void Model::attachView(AbstractView *view)
 */
 void Model::detachView(AbstractView *view, ViewNotification emitDetachNotify)
 {
-//    Internal::WriteLocker locker(m_d);
+//    Internal::WriteLocker locker(d);
     bool emitNotify = (emitDetachNotify == NotifyView);
 
     RewriterView *rewriterView = qobject_cast<RewriterView*>(view);
     if (rewriterView) {
-        m_d->setRewriterView(0);
+        d->setRewriterView(0);
         return;
     }
 
     NodeInstanceView *nodeInstanceView = qobject_cast<NodeInstanceView*>(view);
     if (nodeInstanceView) {
-        m_d->setNodeInstanceView(0);
+        d->setNodeInstanceView(0);
         return;
     }
 
-    m_d->detachView(view, emitNotify);
+    d->detachView(view, emitNotify);
 }
 
 
