@@ -2,49 +2,55 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2010 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
 #include "texteditoroverlay.h"
+#include "basetexteditor.h"
 
+#include <QtCore/QDebug>
+#include <QtCore/QMap>
 #include <QtGui/QPainter>
 #include <QtGui/QTextBlock>
 
 using namespace TextEditor;
 using namespace TextEditor::Internal;
 
-
-
-TextEditorOverlay::TextEditorOverlay(BaseTextEditor *editor)
-    :QObject(editor) {
-    m_visible = false;
-    m_borderWidth = 1;
-    m_dropShadowWidth = 2;
-    m_editor = editor;
-    m_alpha = true;
-    m_viewport = editor->viewport();
+TextEditorOverlay::TextEditorOverlay(BaseTextEditorWidget *editor) :
+    QObject(editor),
+    m_visible(false),
+    m_borderWidth(1),
+    m_dropShadowWidth(2),
+    m_alpha(true),
+    m_firstSelectionOriginalBegin(-1),
+    m_editor(editor),
+    m_viewport(editor->viewport())
+{
 }
 
 void TextEditorOverlay::update()
@@ -59,7 +65,8 @@ void TextEditorOverlay::setVisible(bool b)
     if (m_visible == b)
         return;
     m_visible = b;
-    m_viewport->update();
+    if (!m_selections.isEmpty())
+        m_viewport->update();
 }
 
 void TextEditorOverlay::clear()
@@ -67,13 +74,13 @@ void TextEditorOverlay::clear()
     if (m_selections.isEmpty())
         return;
     m_selections.clear();
+    m_firstSelectionOriginalBegin = -1;
     update();
 }
 
 void TextEditorOverlay::addOverlaySelection(int begin, int end,
                                             const QColor &fg, const QColor &bg,
-                                            uint overlaySelectionFlags,
-                                            int verticalBlockSelection)
+                                            uint overlaySelectionFlags)
 {
     if (end < begin)
         return;
@@ -84,22 +91,24 @@ void TextEditorOverlay::addOverlaySelection(int begin, int end,
     selection.m_fg = fg;
     selection.m_bg = bg;
 
-    if (overlaySelectionFlags & ExpandBegin) {
-        if (begin > 0 && begin < end) { // not empty
-            selection.m_expandBegin = true;
-        }
-    }
-
     selection.m_cursor_begin = QTextCursor(document->docHandle(), begin);
     selection.m_cursor_end = QTextCursor(document->docHandle(), end);
 
+    if (overlaySelectionFlags & ExpandBegin) {
+        if (begin > 0 && begin < end) { // not empty
+            selection.m_cursor_begin.setKeepPositionOnInsert(true);
+        }
+    }
+
     if (overlaySelectionFlags & LockSize)
         selection.m_fixedLength = (end - begin);
 
-
     selection.m_dropShadow = (overlaySelectionFlags & DropShadow);
 
-    selection.m_verticalBlockSelection = verticalBlockSelection;
+    if (m_selections.isEmpty())
+        m_firstSelectionOriginalBegin = begin;
+    else if (begin < m_firstSelectionOriginalBegin)
+        qWarning() << "overlay selections not in order";
 
     m_selections.append(selection);
     update();
@@ -108,11 +117,9 @@ void TextEditorOverlay::addOverlaySelection(int begin, int end,
 
 void TextEditorOverlay::addOverlaySelection(const QTextCursor &cursor,
                                             const QColor &fg, const QColor &bg,
-                                            uint overlaySelectionFlags,
-                                            int verticalBlockSelection)
+                                            uint overlaySelectionFlags)
 {
-    addOverlaySelection(cursor.selectionStart(), cursor.selectionEnd(), fg, bg, overlaySelectionFlags,
-                        verticalBlockSelection);
+    addOverlaySelection(cursor.selectionStart(), cursor.selectionEnd(), fg, bg, overlaySelectionFlags);
 }
 
 QRect TextEditorOverlay::rect() const
@@ -121,7 +128,7 @@ QRect TextEditorOverlay::rect() const
 }
 
 QPainterPath TextEditorOverlay::createSelectionPath(const QTextCursor &begin, const QTextCursor &end,
-                                                    const QRect &clip, int verticalBlockSelection)
+                                                    const QRect &clip)
 {
     if (begin.isNull() || end.isNull() || begin.position() > end.position())
         return QPainterPath();
@@ -169,17 +176,17 @@ QPainterPath TextEditorOverlay::createSelectionPath(const QTextCursor &begin, co
 
             int beginChar = 0;
             if (!inSelection) {
-                beginChar = begin.positionInBlock();
-                line = blockLayout->lineForTextPosition(beginChar);
+                if (block == begin.block()) {
+                    beginChar = begin.positionInBlock();
+                    line = blockLayout->lineForTextPosition(beginChar);
+                    firstOrLastBlock = true;
+                }
                 inSelection = true;
-                firstOrLastBlock = true;
-            } else if (verticalBlockSelection) {
-                beginChar = qMin(block.length()-1, begin.positionInBlock());
             } else {
-                while (beginChar < block.length() && document->characterAt(block.position() + beginChar).isSpace())
-                    ++beginChar;
-                if (beginChar == block.length())
-                    beginChar = 0;
+//                while (beginChar < block.length() && document->characterAt(block.position() + beginChar).isSpace())
+//                    ++beginChar;
+//                if (beginChar == block.length())
+//                    beginChar = 0;
             }
 
             int lastLine = blockLayout->lineCount()-1;
@@ -189,8 +196,6 @@ QPainterPath TextEditorOverlay::createSelectionPath(const QTextCursor &begin, co
                 lastLine = blockLayout->lineForTextPosition(endChar).lineNumber();
                 inSelection = false;
                 firstOrLastBlock = true;
-            } else if (verticalBlockSelection) {
-                endChar = qMin(block.length()-1, begin.positionInBlock() + verticalBlockSelection);
             } else {
                 endChar = block.length();
                 while (endChar > beginChar && document->characterAt(block.position() + endChar - 1).isSpace())
@@ -211,7 +216,7 @@ QPainterPath TextEditorOverlay::createSelectionPath(const QTextCursor &begin, co
                         lineRect.setRight(line.cursorToX(endChar));
                     selection += lineRect.translated(blockGeometry.topLeft());
                 }
-            } else if (!verticalBlockSelection){ // empty lines
+            } else { // empty lines
                 const int emptyLineSelectionSize = 16;
                 if (!firstOrLastBlock && !selection.isEmpty()) { // middle
                     lineRect.setLeft(selection.last().left());
@@ -315,8 +320,6 @@ void TextEditorOverlay::paintSelection(QPainter *painter,
 {
 
     QTextCursor begin = selection.m_cursor_begin;
-    if (selection.m_expandBegin)
-        begin.setPosition(begin.position() + 1);
 
     const QTextCursor &end= selection.m_cursor_end;
     const QColor &fg = selection.m_fg;
@@ -328,8 +331,7 @@ void TextEditorOverlay::paintSelection(QPainter *painter,
         || begin.position() > end.position())
         return;
 
-    QPainterPath path = createSelectionPath(begin, end, m_editor->viewport()->rect(),
-                                            selection.m_verticalBlockSelection);
+    QPainterPath path = createSelectionPath(begin, end, m_editor->viewport()->rect());
 
     painter->save();
     QColor penColor = fg;
@@ -389,8 +391,7 @@ void TextEditorOverlay::fillSelection(QPainter *painter,
     if (begin.isNull() || end.isNull() || begin.position() > end.position())
         return;
 
-    QPainterPath path = createSelectionPath(begin, end, m_editor->viewport()->rect(),
-                                            selection.m_verticalBlockSelection);
+    QPainterPath path = createSelectionPath(begin, end, m_editor->viewport()->rect());
 
     painter->save();
     painter->translate(-.5, -.5);
@@ -453,13 +454,90 @@ void TextEditorOverlay::fill(QPainter *painter, const QColor &color, const QRect
     }
 }
 
+/*! \returns true if any selection contains \a cursor, where a cursor on the
+             start or end of a selection is counted as contained.
+*/
 bool TextEditorOverlay::hasCursorInSelection(const QTextCursor &cursor) const
+{
+    if (selectionIndexForCursor(cursor) != -1)
+        return true;
+    return false;
+}
+
+int TextEditorOverlay::selectionIndexForCursor(const QTextCursor &cursor) const
 {
     for (int i = 0; i < m_selections.size(); ++i) {
         const OverlaySelection &selection = m_selections.at(i);
-        if (cursor.position() > selection.m_cursor_begin.position()
+        if (cursor.position() >= selection.m_cursor_begin.position()
             && cursor.position() <= selection.m_cursor_end.position())
-            return true;
+            return i;
     }
-    return false;
+    return -1;
+}
+
+QString TextEditorOverlay::selectionText(int selectionIndex) const
+{
+    return assembleCursorForSelection(selectionIndex).selectedText();
+}
+
+QTextCursor TextEditorOverlay::assembleCursorForSelection(int selectionIndex) const
+{
+    const OverlaySelection &selection = m_selections.at(selectionIndex);
+    QTextCursor cursor(m_editor->document());
+    cursor.setPosition(selection.m_cursor_begin.position());
+    cursor.setPosition(selection.m_cursor_end.position(), QTextCursor::KeepAnchor);
+    return cursor;
+}
+
+void TextEditorOverlay::mapEquivalentSelections()
+{
+    m_equivalentSelections.clear();
+    m_equivalentSelections.resize(m_selections.size());
+
+    QMap<QString, int> all;
+    for (int i = 0; i < m_selections.size(); ++i)
+        all.insertMulti(selectionText(i), i);
+
+    const QList<QString> &uniqueKeys = all.uniqueKeys();
+    foreach (const QString &key, uniqueKeys) {
+        QList<int> indexes;
+        QMap<QString, int>::const_iterator lbit = all.lowerBound(key);
+        QMap<QString, int>::const_iterator ubit = all.upperBound(key);
+        while (lbit != ubit) {
+            indexes.append(lbit.value());
+            ++lbit;
+        }
+
+        foreach (int index, indexes)
+            m_equivalentSelections[index] = indexes;
+    }
+}
+
+void TextEditorOverlay::updateEquivalentSelections(const QTextCursor &cursor)
+{
+    int selectionIndex = selectionIndexForCursor(cursor);
+    if (selectionIndex == -1)
+        return;
+
+    const QString &currentText = selectionText(selectionIndex);
+    const QList<int> &equivalents = m_equivalentSelections.at(selectionIndex);
+    foreach (int i, equivalents) {
+        if (i == selectionIndex)
+            continue;
+        const QString &equivalentText = selectionText(i);
+        if (currentText != equivalentText) {
+            QTextCursor selectionCursor = assembleCursorForSelection(i);
+            selectionCursor.joinPreviousEditBlock();
+            selectionCursor.removeSelectedText();
+            selectionCursor.insertText(currentText);
+            selectionCursor.endEditBlock();
+        }
+    }
+}
+
+bool TextEditorOverlay::hasFirstSelectionBeginMoved() const
+{
+    if (m_firstSelectionOriginalBegin == -1 || m_selections.isEmpty())
+        return false;
+    return m_selections.at(0).m_cursor_begin.position() != m_firstSelectionOriginalBegin;
 }
